@@ -18,59 +18,72 @@
 #include "s2/mutable_s2shape_index.h"
 
 #include <algorithm>
-#include <functional>
+#include <cmath>
+#include <cstddef>
 #include <memory>
 #include <numeric>
 #include <thread>
+#include <cstdint>
 #include <string>
 #include <utility>
 #include <vector>
 
 #include <gtest/gtest.h>
 
-#include "absl/flags/reflection.h"
+#include "absl/flags/declare.h"
 #include "absl/flags/flag.h"
-#include "absl/memory/memory.h"
+#include "absl/flags/reflection.h"
+#include "absl/log/absl_check.h"
+#include "absl/log/absl_log.h"
+#include "absl/log/log_streamer.h"
+#include "absl/random/random.h"
+#include "absl/strings/str_cat.h"
 #include "absl/strings/str_format.h"
+#include "absl/types/span.h"
 
-#include "s2/base/commandlineflags.h"
-#include "s2/base/logging.h"
 #include "s2/base/log_severity.h"
 #include "s2/r2.h"
 #include "s2/r2rect.h"
 #include "s2/s1angle.h"
-#include "s2/s2cap.h"
 #include "s2/s2cell.h"
 #include "s2/s2cell_id.h"
 #include "s2/s2cell_union.h"
-#include "s2/s2debug.h"
+#include "s2/s2coords.h"
 #include "s2/s2edge_clipping.h"
-#include "s2/s2edge_crosser.h"
 #include "s2/s2edge_distances.h"
 #include "s2/s2edge_vector_shape.h"
 #include "s2/s2error.h"
+#include "s2/s2fractal.h"
 #include "s2/s2lax_polygon_shape.h"
+#include "s2/s2lax_polyline_shape.h"
 #include "s2/s2loop.h"
+#include "s2/s2padded_cell.h"
+#include "s2/s2point.h"
 #include "s2/s2point_vector_shape.h"
 #include "s2/s2pointutil.h"
 #include "s2/s2polygon.h"
+#include "s2/s2polyline.h"
+#include "s2/s2random.h"
+#include "s2/s2shape.h"
+#include "s2/s2shape_index.h"
 #include "s2/s2shapeutil_coding.h"
 #include "s2/s2shapeutil_contains_brute_force.h"
 #include "s2/s2shapeutil_testing.h"
-#include "s2/s2shapeutil_visit_crossing_edge_pairs.h"
 #include "s2/s2testing.h"
 #include "s2/s2text_format.h"
 #include "s2/thread_testing.h"
+#include "s2/util/coding/coder.h"
+#include "s2/util/math/matrix3x3.h"
+#include "s2/util/random/shared_bit_gen.h"
 
-using absl::make_unique;
 using absl::WrapUnique;
 using s2textformat::MakePolylineOrDie;
+using std::make_unique;
 using std::string;
 using std::unique_ptr;
 using std::vector;
 
-S2_DECLARE_double(s2shape_index_min_short_edge_fraction);
-S2_DECLARE_double(s2shape_index_cell_size_to_long_edge_ratio);
+ABSL_DECLARE_FLAG(double, s2shape_index_min_short_edge_fraction);
 
 class MutableS2ShapeIndexTest : public ::testing::Test {
  protected:
@@ -98,14 +111,14 @@ class MutableS2ShapeIndexTest : public ::testing::Test {
   using BatchDescriptor = MutableS2ShapeIndex::BatchDescriptor;
 
   // Converts the given vector of batches to a human-readable form.
-  static string ToString(const vector<BatchDescriptor>& batches);
+  static string ToString(absl::Span<const BatchDescriptor> batches);
 
   // Verifies that removing and adding the given combination of shapes with
   // the given memory budget yields the expected vector of batches.
-  void TestBatchGenerator(
-      int num_edges_removed, const vector<int>& shape_edges_added,
-      int64 tmp_memory_budget, int shape_id_begin,
-      const vector<BatchDescriptor>& expected_batches);
+  void TestBatchGenerator(int num_edges_removed,
+                          absl::Span<const int> shape_edges_added,
+                          int64_t tmp_memory_budget, int shape_id_begin,
+                          absl::Span<const BatchDescriptor> expected_batches);
 };
 
 void MutableS2ShapeIndexTest::QuadraticValidate() {
@@ -222,7 +235,7 @@ void MutableS2ShapeIndexTest::TestEncodeDecode() {
 }
 
 /*static*/ string MutableS2ShapeIndexTest::ToString(
-    const vector<BatchDescriptor>& batches) {
+    absl::Span<const BatchDescriptor> batches) {
   string result;
   for (const auto& batch : batches) {
     if (!result.empty()) result += ", ";
@@ -235,9 +248,9 @@ void MutableS2ShapeIndexTest::TestEncodeDecode() {
 }
 
 void MutableS2ShapeIndexTest::TestBatchGenerator(
-    int num_edges_removed, const vector<int>& shape_edges_added,
-    int64 tmp_memory_budget, int shape_id_begin,
-    const vector<BatchDescriptor>& expected_batches) {
+    int num_edges_removed, absl::Span<const int> shape_edges_added,
+    int64_t tmp_memory_budget, int shape_id_begin,
+    absl::Span<const BatchDescriptor> expected_batches) {
   absl::FlagSaver fs;
   absl::SetFlag(&FLAGS_s2shape_index_tmp_memory_budget, tmp_memory_budget);
 
@@ -267,7 +280,7 @@ void TestIteratorMethods(const MutableS2ShapeIndex& index) {
     auto skipped = S2CellUnion::FromBeginEnd(min_cellid, cellid.range_min());
     for (S2CellId skipped_id : skipped) {
       EXPECT_FALSE(it2.Locate(skipped_id.ToPoint()));
-      EXPECT_EQ(S2ShapeIndex::DISJOINT, it2.Locate(skipped_id));
+      EXPECT_EQ(S2CellRelation::DISJOINT, it2.Locate(skipped_id));
       it2.Begin();
       it2.Seek(skipped_id);
       EXPECT_EQ(cellid, it2.id());
@@ -282,22 +295,22 @@ void TestIteratorMethods(const MutableS2ShapeIndex& index) {
       EXPECT_EQ(ids.back(), it2.id());
     }
     it2.Begin();
-    EXPECT_EQ(cellid.ToPoint(), it.center());
-    EXPECT_TRUE(it2.Locate(it.center()));
+    EXPECT_EQ(cellid.ToPoint(), it.id().ToPoint());
+    EXPECT_TRUE(it2.Locate(it.id().ToPoint()));
     EXPECT_EQ(cellid, it2.id());
     it2.Begin();
-    EXPECT_EQ(S2ShapeIndex::INDEXED, it2.Locate(cellid));
+    EXPECT_EQ(S2CellRelation::INDEXED, it2.Locate(cellid));
     EXPECT_EQ(cellid, it2.id());
     if (!cellid.is_face()) {
       it2.Begin();
-      EXPECT_EQ(S2ShapeIndex::SUBDIVIDED, it2.Locate(cellid.parent()));
+      EXPECT_EQ(S2CellRelation::SUBDIVIDED, it2.Locate(cellid.parent()));
       EXPECT_LE(it2.id(), cellid);
       EXPECT_GE(it2.id(), cellid.parent().range_min());
     }
     if (!cellid.is_leaf()) {
       for (int i = 0; i < 4; ++i) {
         it2.Begin();
-        EXPECT_EQ(S2ShapeIndex::INDEXED, it2.Locate(cellid.child(i)));
+        EXPECT_EQ(S2CellRelation::INDEXED, it2.Locate(cellid.child(i)));
         EXPECT_EQ(cellid, it2.id());
       }
     }
@@ -465,7 +478,7 @@ TEST_F(MutableS2ShapeIndexTest, ShrinkToFitOptimization) {
 
 TEST_F(MutableS2ShapeIndexTest, LoopsSpanningThreeFaces) {
   S2Polygon polygon;
-  const int kNumEdges = 100;  // Validation is quadratic
+  constexpr int kNumEdges = 100;  // Validation is quadratic
   // Construct two loops consisting of kNumEdges vertices each, centered
   // around the cube vertex at the start of the Hilbert curve.
   S2Testing::ConcentricLoopsPolygon(S2Point(1, -1, -1).Normalize(), 2,
@@ -480,7 +493,7 @@ TEST_F(MutableS2ShapeIndexTest, LoopsSpanningThreeFaces) {
 }
 
 TEST_F(MutableS2ShapeIndexTest, ManyIdenticalEdges) {
-  const int kNumEdges = 100;  // Validation is quadratic
+  constexpr int kNumEdges = 100;  // Validation is quadratic
   S2Point a = S2Point(0.99, 0.99, 1).Normalize();
   S2Point b = S2Point(-0.99, -0.99, 1).Normalize();
   for (int i = 0; i < kNumEdges; ++i) {
@@ -520,7 +533,7 @@ TEST_F(MutableS2ShapeIndexTest, DegenerateEdge) {
 TEST_F(MutableS2ShapeIndexTest, ManyTinyEdges) {
   // This test adds many edges to a single leaf cell, to check that
   // subdivision stops when no further subdivision is possible.
-  const int kNumEdges = 100;  // Validation is quadratic
+  constexpr int kNumEdges = 100;  // Validation is quadratic
   // Construct two points in the same leaf cell.
   S2Point a = S2CellId(S2Point(1, 0, 0)).ToPoint();
   S2Point b = (a + S2Point(0, 1e-12, 0)).Normalize();
@@ -564,7 +577,8 @@ TEST_F(MutableS2ShapeIndexTest, RandomUpdates) {
   absl::SetFlag(&FLAGS_s2shape_index_tmp_memory_budget, 10000);
 
   // Allow the seed to be varied from the command line.
-  S2Testing::rnd.Reset(absl::GetFlag(FLAGS_s2_random_seed));
+  absl::BitGen bitgen(S2Testing::MakeTaggedSeedSeq(
+      "RANDOM_UPDATES", absl::LogInfoStreamer(__FILE__, __LINE__).stream()));
 
   // A few polylines.
   index_.Add(make_unique<S2Polyline::OwningShape>(
@@ -606,24 +620,23 @@ TEST_F(MutableS2ShapeIndexTest, RandomUpdates) {
   QuadraticValidate();
   TestEncodeDecode();
   for (int iter = 0; iter < 100; ++iter) {
-    S2_VLOG(1) << "Iteration: " << iter;
+    ABSL_VLOG(1) << "Iteration: " << iter;
     // Choose some shapes to add and release.
-    int num_updates = 1 + S2Testing::rnd.Skewed(5);
+    int num_updates = 1 + s2random::SkewedInt(bitgen, 5);
     for (int n = 0; n < num_updates; ++n) {
-      if (S2Testing::rnd.OneIn(2) && !added.empty()) {
-        int i = S2Testing::rnd.Uniform(added.size());
-        S2_VLOG(1) << "  Released shape " << added[i]
-                << " (" << index_.shape(added[i]) << ")";
+      if (absl::Bernoulli(bitgen, 0.5) && !added.empty()) {
+        size_t i = absl::Uniform<size_t>(bitgen, 0, added.size());
+        ABSL_VLOG(1) << "  Released shape " << added[i] << " ("
+                     << index_.shape(added[i]) << ")";
         released.push_back(index_.Release(added[i]));
         added.erase(added.begin() + i);
       } else if (!released.empty()) {
-        int i = S2Testing::rnd.Uniform(released.size());
+        size_t i = absl::Uniform<size_t>(bitgen, 0, released.size());
         S2Shape* shape = released[i].get();
-        index_.Add(std::move(released[i]));  // Changes shape->id().
+        int shape_id = index_.Add(std::move(released[i]));
         released.erase(released.begin() + i);
-        added.push_back(shape->id());
-        S2_VLOG(1) << "  Added shape " << shape->id()
-                << " (" << shape << ")";
+        added.push_back(shape_id);
+        ABSL_VLOG(1) << "  Added shape " << shape_id << " (" << shape << ")";
       }
     }
     QuadraticValidate();
@@ -640,13 +653,15 @@ TEST_F(MutableS2ShapeIndexTest, RandomUpdates) {
 // reads.
 class LazyUpdatesTest : public s2testing::ReaderWriterTest {
  public:
-  LazyUpdatesTest() {}
+  LazyUpdatesTest() = default;
 
   void WriteOp() override {
+    util_random::SharedBitGen bitgen;
     index_.Clear();
-    int num_vertices = 4 * S2Testing::rnd.Skewed(10);  // Up to 4K vertices
+    int num_vertices =
+        4 * s2random::SkewedInt(bitgen, 10);  // Up to 4K vertices
     unique_ptr<S2Loop> loop(S2Loop::MakeRegularLoop(
-        S2Testing::RandomPoint(), S2Testing::KmToAngle(5), num_vertices));
+        s2random::Point(bitgen), S2Testing::KmToAngle(5), num_vertices));
     index_.Add(make_unique<S2Loop::OwningShape>(std::move(loop)));
   }
 
@@ -669,8 +684,8 @@ TEST(MutableS2ShapeIndex, ConstMethodsThreadSafe) {
 
   // The number of readers should be large enough so that it is likely that
   // several readers will be running at once (with a multiple-core CPU).
-  const int kNumReaders = 8;
-  const int kIters = 100;
+  constexpr int kNumReaders = 8;
+  constexpr int kIters = 100;
   test.Run(kNumReaders, kIters);
 }
 
@@ -691,7 +706,7 @@ TEST(MutableS2ShapeIndex, MixedGeometry) {
   index.Add(make_unique<S2Loop::Shape>(&loop));
   MutableS2ShapeIndex::Iterator it(&index);
   // No geometry intersects face 1, so there should be no index cells there.
-  EXPECT_EQ(S2ShapeIndex::DISJOINT, it.Locate(S2CellId::FromFace(1)));
+  EXPECT_EQ(S2CellRelation::DISJOINT, it.Locate(S2CellId::FromFace(1)));
 }
 
 TEST_F(MutableS2ShapeIndexTest, LinearSpace) {
@@ -712,7 +727,7 @@ TEST_F(MutableS2ShapeIndexTest, LinearSpace) {
   // is the max_edges_per_cell() parameter.  The edges are divided such that
   // there are equal numbers of long and short edges; this maximizes the index
   // size when FLAGS_s2shape_index_min_short_edge_fraction is set to zero.
-  const int kNumEdges = 100;  // Validation is quadratic
+  constexpr int kNumEdges = 100;  // Validation is quadratic
   int edges_per_cluster = options.max_edges_per_cell() + 1;
   int num_clusters = (kNumEdges / 2) / edges_per_cluster;
 
@@ -766,12 +781,43 @@ TEST_F(MutableS2ShapeIndexTest, LongIndexEntriesBound) {
   }
   int sum = 0;
   for (int i = 0; i < counts.size(); ++i) {
-    S2_LOG(INFO) << i << ": " << counts[i];
+    ABSL_LOG(INFO) << i << ": " << counts[i];
     sum += counts[i];
   }
   EXPECT_EQ(sum, 366);
 }
 
+TEST_F(MutableS2ShapeIndexTest, AddRemoveShapeContainingOrigin) {
+  // Add a shape that contains InteriorTracker::Origin().
+  S2Point origin = S2::FaceUVtoXYZ(0, -1, -1).Normalize();
+  index_.Add(make_unique<S2Loop::OwningShape>(
+      S2Loop::MakeRegularLoop(origin, S1Angle::Degrees(1), 10)));
+
+  // Force an index update.
+  index_.ForceBuild();
+
+  // Remove the shape.
+  index_.Remove(0);
+
+  // Force another index update.
+  index_.ForceBuild();
+
+  // The iterator should be empty.
+  MutableS2ShapeIndex::Iterator it(&index_, S2ShapeIndex::BEGIN);
+  EXPECT_TRUE(it.done());
+}
+
+TEST_F(MutableS2ShapeIndexTest, DecoderCatchesInvalidIndex) {
+  // An S2Shape index with one face cell but no actual shapes encoded.
+  const std::string encoded{"E\000P\340\010\020\000"};
+
+  Decoder decoder(encoded.data(), encoded.size());
+  MutableS2ShapeIndex index;
+
+  bool ok = index.Init(&decoder, s2shapeutil::FullDecodeShapeFactory(&decoder));
+  EXPECT_FALSE(ok);
+}
+
 TEST(S2Shape, user_data) {
   struct MyData {
     int x, y;
@@ -790,9 +836,9 @@ TEST(S2Shape, user_data) {
   };
   MyEdgeVectorShape shape(MyData(3, 5));
   MyData* data = static_cast<MyData*>(shape.mutable_user_data());
-  S2_DCHECK_EQ(3, data->x);
+  ABSL_DCHECK_EQ(3, data->x);
   data->y = 10;
-  S2_DCHECK_EQ(10, static_cast<const MyData*>(shape.user_data())->y);
+  ABSL_DCHECK_EQ(10, static_cast<const MyData*>(shape.user_data())->y);
 }
 
 }  // namespace
