@@ -50,11 +50,15 @@
 
 #include <algorithm>
 #include <cmath>
+#include <iterator>
 #include <memory>
 #include <utility>
 #include <vector>
 
-#include "absl/memory/memory.h"
+#include "absl/log/absl_check.h"
+#include "absl/types/span.h"
+#include "s2/s1angle.h"
+#include "s2/s1chord_angle.h"
 #include "s2/s2builder.h"
 #include "s2/s2builder_layer.h"
 #include "s2/s2builderutil_snap_functions.h"
@@ -64,18 +68,26 @@
 #include "s2/s2edge_distances.h"
 #include "s2/s2error.h"
 #include "s2/s2lax_loop_shape.h"
+#include "s2/s2memory_tracker.h"
+#include "s2/s2point.h"
+#include "s2/s2point_span.h"
+#include "s2/s2pointutil.h"
+#include "s2/s2predicates.h"
 #include "s2/s2predicates_internal.h"
+#include "s2/s2shape.h"
+#include "s2/s2shape_index.h"
 #include "s2/s2shape_measures.h"
 #include "s2/s2shapeutil_contains_brute_force.h"
+#include "s2/s2winding_operation.h"
 #include "s2/util/math/mathutil.h"
 
-using absl::make_unique;
 using s2pred::DBL_ERR;
 using std::ceil;
+using std::make_unique;
 using std::max;
 using std::min;
-using std::vector;
 using std::unique_ptr;
+using std::vector;
 
 // The errors due to buffering can be categorized as follows:
 //
@@ -129,10 +141,6 @@ static constexpr S1Angle kMinRequestedError = S1Angle::Radians(2 * DBL_ERR);
 static constexpr S1Angle kMaxAbsoluteInterpolationError =
     S2::kGetPointOnLineError + S2::kGetPointOnRayPerpendicularError;
 
-// TODO(user, b/210097200): Remove when we require c++17 for opensource.
-constexpr double S2BufferOperation::Options::kMinErrorFraction;
-constexpr double S2BufferOperation::Options::kMaxCircleSegments;
-
 S2BufferOperation::Options::Options()
     : snap_function_(
           make_unique<s2builderutil::IdentitySnapFunction>(S1Angle::Zero())) {
@@ -175,12 +183,12 @@ double S2BufferOperation::Options::error_fraction() const {
 }
 
 void S2BufferOperation::Options::set_error_fraction(double error_fraction) {
-  S2_DCHECK_GE(error_fraction, kMinErrorFraction);
-  S2_DCHECK_LE(error_fraction, 1.0);
+  ABSL_DCHECK_GE(error_fraction, kMinErrorFraction);
+  ABSL_DCHECK_LE(error_fraction, 1.0);
   error_fraction_ = max(kMinErrorFraction, min(1.0, error_fraction));
 }
 
-const S1Angle S2BufferOperation::Options::max_error() const {
+S1Angle S2BufferOperation::Options::max_error() const {
   // See comments for kMinRequestedError above.
   S2Builder::Options builder_options(*snap_function_);
   builder_options.set_split_crossing_edges(true);
@@ -199,8 +207,8 @@ double S2BufferOperation::Options::circle_segments() const {
 }
 
 void S2BufferOperation::Options::set_circle_segments(double circle_segments) {
-  S2_DCHECK_GE(circle_segments, 2.0);
-  S2_DCHECK_LE(circle_segments, kMaxCircleSegments);
+  ABSL_DCHECK_GE(circle_segments, 2.0);
+  ABSL_DCHECK_LE(circle_segments, kMaxCircleSegments);
   circle_segments = max(2.0, min(kMaxCircleSegments, circle_segments));
 
   // We convert circle_segments to error_fraction using planar geometry,
@@ -264,15 +272,14 @@ void S2BufferOperation::Options::set_memory_tracker(S2MemoryTracker* tracker) {
   memory_tracker_ = tracker;
 }
 
-S2BufferOperation::S2BufferOperation() {
-}
+S2BufferOperation::S2BufferOperation() = default;
 
 S2BufferOperation::S2BufferOperation(unique_ptr<S2Builder::Layer> result_layer,
                                      const Options& options) {
   Init(std::move(result_layer), options);
 }
 
-void S2BufferOperation::Init(std::unique_ptr<S2Builder::Layer> result_layer,
+void S2BufferOperation::Init(unique_ptr<S2Builder::Layer> result_layer,
                              const Options& options) {
   options_ = options;
   ref_point_ = S2::Origin();
@@ -339,7 +346,7 @@ S1Angle S2BufferOperation::GetMaxEdgeSpan(S1Angle radius,
   // we like, however we always use at least 3 edges to approximate a circle.
   S1Angle step = S1Angle::Radians(2 * M_PI / 3 + 1e-15);
   S1Angle min_radius = radius - requested_error;
-  S2_DCHECK_GE(min_radius, S1Angle::Zero());
+  ABSL_DCHECK_GE(min_radius, S1Angle::Zero());
   if (radius.radians() < M_PI_2) {
     step = min(step, S1Angle::Radians(2 * acos(tan(min_radius) / tan(radius))));
   } else if (min_radius.radians() > M_PI_2) {
@@ -354,7 +361,7 @@ S1Angle S2BufferOperation::GetMaxEdgeSpan(S1Angle radius,
 // winding number of the reference point if necessary.
 void S2BufferOperation::SetInputVertex(const S2Point& new_a) {
   if (have_input_start_) {
-    S2_DCHECK(have_offset_start_);
+    ABSL_DCHECK(have_offset_start_);
     UpdateRefWinding(sweep_a_, sweep_b_, new_a);
   } else {
     input_start_ = new_a;
@@ -370,7 +377,7 @@ void S2BufferOperation::AddOffsetVertex(const S2Point& new_b) {
   if (!tracker_.AddSpace(&path_, 1)) return;
   path_.push_back(new_b);
   if (have_offset_start_) {
-    S2_DCHECK(have_input_start_);
+    ABSL_DCHECK(have_input_start_);
     UpdateRefWinding(sweep_a_, sweep_b_, new_b);
   } else {
     offset_start_ = new_b;
@@ -474,7 +481,7 @@ void S2BufferOperation::AddPoint(const S2Point& point) {
 // of AB if buffer_sign_ < 0.
 inline S2Point S2BufferOperation::GetEdgeAxis(const S2Point& a,
                                               const S2Point& b) const {
-  S2_DCHECK_NE(buffer_sign_, 0);
+  ABSL_DCHECK_NE(buffer_sign_, 0);
   return buffer_sign_ * S2::RobustCrossProd(b, a).Normalize();
 }
 
@@ -537,9 +544,9 @@ void S2BufferOperation::CloseEdgeArc(const S2Point& a, const S2Point& b) {
 // not leave the valid buffer zone of the previous edge and vertex.
 void S2BufferOperation::BufferEdgeAndVertex(const S2Point& a, const S2Point& b,
                                             const S2Point& c) {
-  S2_DCHECK_NE(a, b);
-  S2_DCHECK_NE(b, c);
-  S2_DCHECK_NE(buffer_sign_, 0);
+  ABSL_DCHECK_NE(a, b);
+  ABSL_DCHECK_NE(b, c);
+  ABSL_DCHECK_NE(buffer_sign_, 0);
   if (!tracker_.ok()) return;
 
   // For left (convex) turns we need to add an offset arc.  For right
@@ -578,7 +585,7 @@ void S2BufferOperation::AddStartCap(const S2Point& a, const S2Point& b) {
       AddOffsetVertex(S2::GetPointOnRay(a, -axis, abs_radius_));
     }
   } else {
-    S2_DCHECK(options_.end_cap_style() == EndCapStyle::ROUND);
+    ABSL_DCHECK(options_.end_cap_style() == EndCapStyle::ROUND);
     if (options_.polyline_side() == PolylineSide::BOTH) {
       // The end cap consists of a semicircle.
       AddVertexArc(a, -axis, axis);
@@ -597,7 +604,7 @@ void S2BufferOperation::AddEndCap(const S2Point& a, const S2Point& b) {
   if (options_.end_cap_style() == EndCapStyle::FLAT) {
     CloseEdgeArc(a, b);  // Close the previous semi-open edge arc if necessary.
   } else {
-    S2_DCHECK(options_.end_cap_style() == EndCapStyle::ROUND);
+    ABSL_DCHECK(options_.end_cap_style() == EndCapStyle::ROUND);
     if (options_.polyline_side() == PolylineSide::BOTH) {
       // The end cap consists of a semicircle.
       AddVertexArc(b, axis, -axis);
@@ -633,7 +640,7 @@ void S2BufferOperation::BufferLoop(S2PointLoopSpan loop) {
     path_.assign(loop.begin(), loop.end());
   } else {
     SetInputVertex(loop[0]);
-    for (int i = 0; i < loop.size(); ++i) {
+    for (size_t i = 0; i < loop.size(); ++i) {
       BufferEdgeAndVertex(loop[i], loop[i + 1], loop[i + 2]);
     }
     CloseBufferRegion();
@@ -751,8 +758,8 @@ void S2BufferOperation::AddShapeIndex(const S2ShapeIndex& index) {
 
 bool S2BufferOperation::Build(S2Error* error) {
   if (buffer_sign_ < 0 && num_polygon_layers_ > 1) {
-    error->Init(S2Error::FAILED_PRECONDITION,
-                "Negative buffer radius requires at most one polygon layer");
+    *error = S2Error::FailedPrecondition(
+        "Negative buffer radius requires at most one polygon layer");
     return false;
   }
   return op_.Build(ref_point_, ref_winding_,
