@@ -22,25 +22,32 @@
 #define S2_S2BUILDER_H_
 
 #include <algorithm>
+#include <cstdint>
 #include <functional>
 #include <memory>
 #include <utility>
 #include <vector>
 
 #include "absl/base/macros.h"
+#include "absl/container/flat_hash_set.h"
+#include "absl/log/absl_check.h"
+#include "absl/log/absl_log.h"
+#include "absl/types/span.h"
 
-#include "s2/base/integral_types.h"
-#include "s2/_fp_contract_off.h"
+#include "s2/_fp_contract_off.h"  // IWYU pragma: keep
 #include "s2/id_set_lexicon.h"
 #include "s2/mutable_s2shape_index.h"
 #include "s2/s1angle.h"
 #include "s2/s1chord_angle.h"
 #include "s2/s2cell_id.h"
+#include "s2/s2edge_crossings.h"
 #include "s2/s2edge_distances.h"
 #include "s2/s2error.h"
 #include "s2/s2memory_tracker.h"
+#include "s2/s2point.h"
 #include "s2/s2point_index.h"
 #include "s2/s2point_span.h"
+#include "s2/s2shape.h"
 #include "s2/s2shape_index.h"
 #include "s2/util/gtl/compact_array.h"
 
@@ -131,11 +138,11 @@ class S2Polyline;
 //  using s2builderutil::IntLatLngSnapFunction;
 //  S2Builder builder(S2Builder::Options(IntLatLngSnapFunction(7)));
 //  S2Polygon output;
-//  builder.StartLayer(absl::make_unique<s2builderutil::S2PolygonLayer>(&output));
+//  builder.StartLayer(std::make_unique<s2builderutil::S2PolygonLayer>(&output));
 //  builder.AddPolygon(input);
 //  S2Error error;
 //  if (!builder.Build(&error)) {
-//    S2_LOG(ERROR) << error;
+//    ABSL_LOG(ERROR) << error;
 //    ...
 //  }
 class S2Builder {
@@ -162,7 +169,7 @@ class S2Builder {
   // holes, the outer loops ("shells") should be directed counter-clockwise
   // while the inner loops ("holes") should be directed clockwise.  Note that
   // S2Builder::AddPolygon() follows this convention automatically.
-  enum class EdgeType : uint8 { DIRECTED, UNDIRECTED };
+  enum class EdgeType : uint8_t { DIRECTED, UNDIRECTED };
 
   // A SnapFunction restricts the locations of the output vertices.  For
   // example, there are predefined snap functions that require vertices to be
@@ -226,7 +233,7 @@ class S2Builder {
   //    crosses an edge.
   class SnapFunction {
    public:
-    virtual ~SnapFunction() {}
+    virtual ~SnapFunction() = default;
 
     // The maximum distance that vertices can move when snapped.  The snap
     // radius can be any value between zero and SnapFunction::kMaxSnapRadius().
@@ -274,10 +281,10 @@ class S2Builder {
     //
     // Note that if your input data includes vertices that were created using
     // S2::GetIntersection(), then you should use a "snap_radius" of
-    // at least S2::kIntersectionSnapRadius, e.g. by calling
+    // at least S2::kIntersectionMergeRadius, e.g. by calling
     //
     //  options.set_snap_function(s2builderutil::IdentitySnapFunction(
-    //      S2::kIntersectionSnapRadius));
+    //      S2::kIntersectionMergeRadius));
     //
     // DEFAULT: s2builderutil::IdentitySnapFunction(S1Angle::Zero())
     // [This does no snapping and preserves all input vertices exactly.]
@@ -302,7 +309,7 @@ class S2Builder {
     // AddIntersection() method which allows intersection points to be added
     // selectively.
     //
-    // When this option if true, intersection_tolerance() is automatically set
+    // When this option is true, intersection_tolerance() is automatically set
     // to a minimum of S2::kIntersectionError (see intersection_tolerance()
     // for why this is necessary).  Note that this means that edges can move
     // by up to S2::kIntersectionError even when the specified snap radius is
@@ -326,7 +333,7 @@ class S2Builder {
     bool split_crossing_edges() const;
     void set_split_crossing_edges(bool split_crossing_edges);
 
-    // Specifes the maximum allowable distance between a vertex added by
+    // Specifies the maximum allowable distance between a vertex added by
     // AddIntersection() and the edge(s) that it is intended to snap to.  This
     // method must be called before AddIntersection() can be used.  It has the
     // effect of increasing the snap radius for edges (but not vertices) by
@@ -356,7 +363,7 @@ class S2Builder {
     //
     // However if the intersection tolerance is set to 1 nanometer then the
     // snap radius for edges is increased to 1.000000001 meters ensuring that
-    // both edges snap to a common vertex even in this worst case.  (Tthis
+    // both edges snap to a common vertex even in this worst case.  (This
     // technique does not work if the vertex snap radius is increased as well;
     // it requires edges and vertices to be handled differently.)
     //
@@ -449,7 +456,7 @@ class S2Builder {
     //   S2Error error;
     //   if (!builder.Build(&error)) {
     //     if (error.code() == S2Error::RESOURCE_EXHAUSTED) {
-    //       S2_LOG(ERROR) << error;  // Memory limit exceeded
+    //       ABSL_LOG(ERROR) << error;  // Memory limit exceeded
     //     }
     //   }
     //
@@ -478,9 +485,9 @@ class S2Builder {
     S2MemoryTracker* memory_tracker_ = nullptr;
   };
 
+  class Graph;
   // The following classes are only needed by Layer implementations.
   class GraphOptions;
-  class Graph;
 
   // For output layers that represent polygons, there is an ambiguity inherent
   // in spherical geometry that does not exist in planar geometry.  Namely, if
@@ -541,8 +548,9 @@ class S2Builder {
   // builder.StartLayer(make_unique<s2builderutil::S2PolylineLayer>(&line2)));
   // ... Add edges using builder.AddEdge(), etc ...
   // S2Error error;
-  // S2_CHECK(builder.Build(&error)) << error;  // Builds "line1" & "line2"
+  // ABSL_CHECK(builder.Build(&error)) << error;  // Builds "line1" & "line2"
   class Layer;
+
   void StartLayer(std::unique_ptr<Layer> layer);
 
   // Adds a degenerate edge (representing a point) to the current layer.
@@ -551,24 +559,28 @@ class S2Builder {
   // Adds the given edge to the current layer.
   void AddEdge(const S2Point& v0, const S2Point& v1);
 
-  // Adds the edges in the given polyline.  Note that polylines with 0 or 1
-  // vertices are defined to have no edges.
+  // Adds the edges in the given polyline to the current layer.  Note that
+  // polylines with 0 or 1 vertices are defined to have no edges.
   void AddPolyline(S2PointSpan polyline);
   void AddPolyline(const S2Polyline& polyline);
 
-  // Adds the edges in the given loop.  Note that a loop consisting of one
-  // vertex adds a single degenerate edge.
+  // Adds the edges in the given loop to the current layer.  Note that a loop
+  // consisting of one vertex adds a single degenerate edge.
+  void AddLoop(S2PointLoopSpan loop);
+
+  // Adds the edges in the given loop to the current layer. Loops with a single
+  // vertex are ignored.
   //
   // If the sign() of an S2Loop is negative (i.e. the loop represents a hole
   // within a polygon), the edge directions are automatically reversed to
   // ensure that the polygon interior is always to the left of every edge.
-  void AddLoop(S2PointLoopSpan loop);
   void AddLoop(const S2Loop& loop);
 
-  // Adds the loops in the given polygon.  Loops representing holes have their
-  // edge directions automatically reversed as described for AddLoop().  Note
-  // that this method does not distinguish between the empty and full polygons,
-  // i.e. adding a full polygon has the same effect as adding an empty one.
+  // Adds the loops in the given polygon to the current layer.  Loops
+  // representing holes have their edge directions automatically reversed as
+  // described for AddLoop().  Note that this method does not distinguish
+  // between the empty and full polygons, i.e. adding a full polygon has the
+  // same effect as adding an empty one.
   void AddPolygon(const S2Polygon& polygon);
 
   // Adds the edges of the given shape to the current layer.
@@ -587,7 +599,7 @@ class S2Builder {
   //
   // This method implicitly overrides the idempotent() option, since adding an
   // intersection point implies a desire to have nearby edges snapped to it
-  // even if these edges already satsify the S2Builder output guarantees.
+  // even if these edges already satisfy the S2Builder output guarantees.
   // (Otherwise for example edges would never be snapped to nearby
   // intersection points when the snap radius is zero.)
   //
@@ -680,7 +692,7 @@ class S2Builder {
   // to add and remove labels hierarchically (e.g., polygon 5, loop 2).  Use
   // set_label() and clear_labels() if you need at most one label per edge.
   //
-  using Label = int32;
+  using Label = int32_t;
 
   // Clear the stack of labels.
   void clear_labels();
@@ -728,16 +740,16 @@ class S2Builder {
   // All types associated with the S2Builder inputs are prefixed with "Input".
 
   // Identifies an input vertex.
-  using InputVertexId = int32;
+  using InputVertexId = int32_t;
 
   // Defines an input edge.
   using InputEdge = std::pair<InputVertexId, InputVertexId>;
 
   // Identifies an input edge.
-  using InputEdgeId = int32;
+  using InputEdgeId = int32_t;
 
   // Identifies the set of input edge ids that were snapped to a given edge.
-  using InputEdgeIdSetId = int32;
+  using InputEdgeIdSetId = int32_t;
 
   // Sort key for prioritizing input vertices.  (Note that keys are *not*
   // compared using std::less; see SortInputVertices for details.)
@@ -750,13 +762,13 @@ class S2Builder {
   // than SiteId is the same as Graph::VertexId, but if there are many layers
   // then each Graph may contain only a subset of the sites.  Also see
   // GraphOptions::allow_vertex_filtering().
-  using SiteId = int32;
+  using SiteId = int32_t;
 
   // Defines an output edge.
   using Edge = std::pair<SiteId, SiteId>;
 
   // Identifies an output edge.
-  using EdgeId = int32;
+  using EdgeId = int32_t;
 
   // Identifies an output edge in a particular layer.
   using LayerEdgeId = std::pair<int, EdgeId>;
@@ -795,22 +807,22 @@ class S2Builder {
     bool DoneSiteIndex(const S2PointIndex<SiteId>& index);
 
     bool TallySimplifyEdgeChains(
-        const std::vector<gtl::compact_array<InputVertexId>>& site_vertices,
-        const std::vector<std::vector<Edge>>& layer_edges);
+        absl::Span<const gtl::compact_array<InputVertexId>> site_vertices,
+        absl::Span<const std::vector<Edge>> layer_edges);
 
     bool TallyFilterVertices(int num_sites,
-                             const std::vector<std::vector<Edge>>& layer_edges);
+                             absl::Span<const std::vector<Edge>> layer_edges);
     bool DoneFilterVertices();
 
    private:
     // The amount of non-inline memory used to store edge sites.
-    int64 edge_sites_bytes_ = 0;
+    int64_t edge_sites_bytes_ = 0;
 
     // The amount of memory used by the S2PointIndex for sites.
-    int64 site_index_bytes_ = 0;
+    int64_t site_index_bytes_ = 0;
 
     // The amount of temporary memory used by Graph::FilterVertices().
-    int64 filter_vertices_bytes_ = 0;
+    int64_t filter_vertices_bytes_ = 0;
   };
 
   InputVertexId AddVertex(const S2Point& v);
@@ -828,18 +840,16 @@ class S2Builder {
   void InsertSiteByDistance(SiteId new_site_id, const S2Point& x,
                             gtl::compact_array<SiteId>* sites);
   void AddExtraSites(const MutableS2ShapeIndex& input_edge_index);
-  void MaybeAddExtraSites(InputEdgeId edge_id,
-                          const std::vector<SiteId>& chain,
+  void MaybeAddExtraSites(InputEdgeId edge_id, absl::Span<const SiteId> chain,
                           const MutableS2ShapeIndex& input_edge_index,
-                          gtl::dense_hash_set<InputEdgeId>* edges_to_resnap);
+                          absl::flat_hash_set<InputEdgeId>* edges_to_resnap);
   void AddExtraSite(const S2Point& new_site,
                     const MutableS2ShapeIndex& input_edge_index,
-                    gtl::dense_hash_set<InputEdgeId>* edges_to_resnap);
+                    absl::flat_hash_set<InputEdgeId>* edges_to_resnap);
   S2Point GetSeparationSite(const S2Point& site_to_avoid,
                             const S2Point& v0, const S2Point& v1,
                             InputEdgeId input_edge_id) const;
-  S2Point GetCoverageEndpoint(const S2Point& p, const S2Point& x,
-                              const S2Point& y, const S2Point& n) const;
+  S2Point GetCoverageEndpoint(const S2Point& p, const S2Point& n) const;
   void SnapEdge(InputEdgeId e, std::vector<SiteId>* chain) const;
 
   void BuildLayers();
@@ -864,10 +874,9 @@ class S2Builder {
       std::vector<std::vector<InputEdgeIdSetId>>* layer_input_edge_ids,
       IdSetLexicon* input_edge_id_set_lexicon);
   void MergeLayerEdges(
-      const std::vector<std::vector<Edge>>& layer_edges,
-      const std::vector<std::vector<InputEdgeIdSetId>>& layer_input_edge_ids,
-      std::vector<Edge>* edges,
-      std::vector<InputEdgeIdSetId>* input_edge_ids,
+      absl::Span<const std::vector<Edge>> layer_edges,
+      absl::Span<const std::vector<InputEdgeIdSetId>> layer_input_edge_ids,
+      std::vector<Edge>* edges, std::vector<InputEdgeIdSetId>* input_edge_ids,
       std::vector<int>* edge_layers) const;
   static bool StableLessThan(const Edge& a, const Edge& b,
                              const LayerEdgeId& ai, const LayerEdgeId& bi);
@@ -940,10 +949,10 @@ class S2Builder {
   std::vector<InputEdgeId> layer_begins_;
   std::vector<IsFullPolygonPredicate> layer_is_full_polygon_predicates_;
 
-  // Each input edge has "label set id" (an int32) representing the set of
+  // Each input edge has "label set id" (an int32_t) representing the set of
   // labels attached to that edge.  This vector is populated only if at least
   // one label is used.
-  using LabelSetId = int32;
+  using LabelSetId = int32_t;
   std::vector<LabelSetId> label_set_ids_;
   IdSetLexicon label_set_lexicon_;
 
@@ -992,9 +1001,9 @@ class S2Builder {
 class S2Builder::GraphOptions {
  public:
   using EdgeType = S2Builder::EdgeType;
-  enum class DegenerateEdges : uint8;
-  enum class DuplicateEdges : uint8;
-  enum class SiblingPairs : uint8;
+  enum class DegenerateEdges : uint8_t;
+  enum class DuplicateEdges : uint8_t;
+  enum class SiblingPairs : uint8_t;
 
   // All S2Builder::Layer subtypes should specify GraphOptions explicitly
   // using this constructor, rather than relying on default values.
@@ -1051,7 +1060,7 @@ class S2Builder::GraphOptions {
   //       for algorithms that require an output edge for every input edge.
   //
   // DEFAULT: DegenerateEdges::KEEP
-  enum class DegenerateEdges : uint8 { DISCARD, DISCARD_EXCESS, KEEP };
+  enum class DegenerateEdges : uint8_t { DISCARD, DISCARD_EXCESS, KEEP };
   DegenerateEdges degenerate_edges() const;
   void set_degenerate_edges(DegenerateEdges degenerate_edges);
 
@@ -1062,7 +1071,7 @@ class S2Builder::GraphOptions {
   // input edge ids.
   //
   // DEFAULT: DuplicateEdges::KEEP
-  enum class DuplicateEdges : uint8 { MERGE, KEEP };
+  enum class DuplicateEdges : uint8_t { MERGE, KEEP };
   DuplicateEdges duplicate_edges() const;
   void set_duplicate_edges(DuplicateEdges duplicate_edges);
 
@@ -1127,15 +1136,19 @@ class S2Builder::GraphOptions {
   // happen even with duplicate degenerate edges (e.g. the edges EE7, EE8).
   //
   // DEFAULT: SiblingPairs::KEEP
-  enum class SiblingPairs : uint8 {
-    DISCARD, DISCARD_EXCESS, KEEP, REQUIRE, CREATE
+  enum class SiblingPairs : uint8_t {
+    DISCARD,
+    DISCARD_EXCESS,
+    KEEP,
+    REQUIRE,
+    CREATE
   };
   SiblingPairs sibling_pairs() const;
   void set_sibling_pairs(SiblingPairs sibling_pairs);
 
-  // This is a specialized option that is only needed by clients want to work
-  // with the graphs for multiple layers at the same time (e.g., in order to
-  // check whether the same edge is present in two different graphs).  [Note
+  // This is a specialized option that is only needed by clients that want to
+  // work with the graphs for multiple layers at the same time (e.g., in order
+  // to check whether the same edge is present in two different graphs).  [Note
   // that if you need to do this, usually it is easier just to build a single
   // graph with suitable edge labels.]
   //
@@ -1205,7 +1218,7 @@ inline S1Angle S2Builder::Options::intersection_tolerance() const {
 
 inline void S2Builder::Options::set_intersection_tolerance(
     S1Angle intersection_tolerance) {
-  S2_DCHECK_GE(intersection_tolerance, S1Angle::Zero());
+  ABSL_DCHECK_GE(intersection_tolerance, S1Angle::Zero());
   intersection_tolerance_ = intersection_tolerance;
 }
 
