@@ -20,27 +20,38 @@
 #include <algorithm>
 #include <cfloat>
 #include <cmath>
-#include <cstring>
-#include <iosfwd>
-#include <mutex>
+#include <cstddef>
+#include <cstdint>
+#include <ostream>
+#include <string>
 #include <vector>
 
-#include "absl/base/call_once.h"
 #include "absl/base/casts.h"
+#include "absl/base/optimization.h"
+#include "absl/log/absl_check.h"
+#include "absl/numeric/bits.h"
 #include "absl/strings/str_cat.h"
+#include "absl/strings/string_view.h"
 
-#include "s2/base/integral_types.h"
-#include "s2/base/logging.h"
-#include "s2/util/bits/bits.h"
+#include "s2/util/coding/coder.h"
 #include "s2/r1interval.h"
+#include "s2/r2.h"
+#include "s2/r2rect.h"
+#include "s2/s1angle.h"
 #include "s2/s2coords.h"
+#include "s2/s2coords_internal.h"
+#include "s2/s2error.h"
 #include "s2/s2latlng.h"
+#include "s2/s2point.h"
+
+namespace {
 
 using absl::StrCat;
-using S2::internal::kSwapMask;
+using absl::string_view;
 using S2::internal::kInvertMask;
 using S2::internal::kPosToIJ;
 using S2::internal::kPosToOrientation;
+using S2::internal::kSwapMask;
 using std::fabs;
 using std::max;
 using std::min;
@@ -49,42 +60,32 @@ using std::vector;
 
 // The following lookup tables are used to convert efficiently between an
 // (i,j) cell index and the corresponding position along the Hilbert curve.
-// "lookup_pos" maps 4 bits of "i", 4 bits of "j", and 2 bits representing the
+// "kLookup.pos" maps 4 bits of "i", 4 bits of "j", and 2 bits representing the
 // orientation of the current cell into 8 bits representing the order in which
 // that subcell is visited by the Hilbert curve, plus 2 bits indicating the
 // new orientation of the Hilbert curve within that subcell.  (Cell
 // orientations are represented as combination of kSwapMask and kInvertMask.)
 //
-// "lookup_ij" is an inverted table used for mapping in the opposite
+// "kLookup.ij" is an inverted table used for mapping in the opposite
 // direction.
 //
 // We also experimented with looking up 16 bits at a time (14 bits of position
 // plus 2 of orientation) but found that smaller lookup tables gave better
 // performance.  (2KB fits easily in the primary cache.)
 
+constexpr int kLookupBits = 4;
 
-// Values for these constants are *declared* in the *.h file. Even though
-// the declaration specifies a value for the constant, that declaration
-// is not a *definition* of storage for the value. Because the values are
-// supplied in the declaration, we don't need the values here. Failing to
-// define storage causes link errors for any code that tries to take the
-// address of one of these values.
-const int S2CellId::kFaceBits;
-const int S2CellId::kNumFaces;
-const int S2CellId::kMaxLevel;
-const int S2CellId::kPosBits;
-const int S2CellId::kMaxSize;
+struct LookupTables {
+  uint16_t pos[1 << (2 * kLookupBits + 2)];
+  uint16_t ij[1 << (2 * kLookupBits + 2)];
+};
 
-static const int kLookupBits = 4;
-static uint16 lookup_pos[1 << (2 * kLookupBits + 2)];
-static uint16 lookup_ij[1 << (2 * kLookupBits + 2)];
-
-static void InitLookupCell(int level, int i, int j, int orig_orientation,
-                           int pos, int orientation) {
+constexpr void InitLookupCell(int level, int i, int j, int orig_orientation,
+                              int pos, int orientation, LookupTables& lookup) {
   if (level == kLookupBits) {
     int ij = (i << kLookupBits) + j;
-    lookup_pos[(ij << 2) + orig_orientation] = (pos << 2) + orientation;
-    lookup_ij[(pos << 2) + orig_orientation] = (ij << 2) + orientation;
+    lookup.pos[(ij << 2) + orig_orientation] = (pos << 2) + orientation;
+    lookup.ij[(pos << 2) + orig_orientation] = (ij << 2) + orientation;
   } else {
     level++;
     i <<= 1;
@@ -92,27 +93,31 @@ static void InitLookupCell(int level, int i, int j, int orig_orientation,
     pos <<= 2;
     const int* r = kPosToIJ[orientation];
     InitLookupCell(level, i + (r[0] >> 1), j + (r[0] & 1), orig_orientation,
-                   pos, orientation ^ kPosToOrientation[0]);
+                   pos, orientation ^ kPosToOrientation[0], lookup);
     InitLookupCell(level, i + (r[1] >> 1), j + (r[1] & 1), orig_orientation,
-                   pos + 1, orientation ^ kPosToOrientation[1]);
+                   pos + 1, orientation ^ kPosToOrientation[1], lookup);
     InitLookupCell(level, i + (r[2] >> 1), j + (r[2] & 1), orig_orientation,
-                   pos + 2, orientation ^ kPosToOrientation[2]);
+                   pos + 2, orientation ^ kPosToOrientation[2], lookup);
     InitLookupCell(level, i + (r[3] >> 1), j + (r[3] & 1), orig_orientation,
-                   pos + 3, orientation ^ kPosToOrientation[3]);
+                   pos + 3, orientation ^ kPosToOrientation[3], lookup);
   }
 }
 
-static absl::once_flag flag;
-inline static void MaybeInit() {
-  absl::call_once(flag, []{
-    InitLookupCell(0, 0, 0, 0, 0, 0);
-    InitLookupCell(0, 0, 0, kSwapMask, 0, kSwapMask);
-    InitLookupCell(0, 0, 0, kInvertMask, 0, kInvertMask);
-    InitLookupCell(0, 0, 0, kSwapMask|kInvertMask, 0, kSwapMask|kInvertMask);
-  });
+constexpr LookupTables MakeLookupTables() {
+  LookupTables lookup{};
+  InitLookupCell(0, 0, 0, 0, 0, 0, lookup);
+  InitLookupCell(0, 0, 0, kSwapMask, 0, kSwapMask, lookup);
+  InitLookupCell(0, 0, 0, kInvertMask, 0, kInvertMask, lookup);
+  InitLookupCell(0, 0, 0, kSwapMask | kInvertMask, 0, kSwapMask | kInvertMask,
+                 lookup);
+  return lookup;
 }
 
-S2CellId S2CellId::advance(int64 steps) const {
+constexpr LookupTables kLookup = MakeLookupTables();
+
+}  // namespace
+
+S2CellId S2CellId::advance(int64_t steps) const {
   if (steps == 0) return *this;
 
   // We clamp the number of steps if necessary to ensure that we do not
@@ -121,44 +126,44 @@ S2CellId S2CellId::advance(int64 steps) const {
 
   int step_shift = 2 * (kMaxLevel - level()) + 1;
   if (steps < 0) {
-    int64 min_steps = -static_cast<int64>(id_ >> step_shift);
-    if (steps < min_steps) steps = min_steps;
+    int64_t min_steps = -static_cast<int64_t>(id_ >> step_shift);
+    steps = max(steps, min_steps);
   } else {
-    int64 max_steps = (kWrapOffset + lsb() - id_) >> step_shift;
-    if (steps > max_steps) steps = max_steps;
+    int64_t max_steps = (kWrapOffset + lsb() - id_) >> step_shift;
+    steps = min(steps, max_steps);
   }
   // If steps is negative, then shifting it left has undefined behavior.
-  // Cast to uint64 for a 2's complement answer.
-  return S2CellId(id_ + (static_cast<uint64>(steps) << step_shift));
+  // Cast to uint64_t for a 2's complement answer.
+  return S2CellId(id_ + (static_cast<uint64_t>(steps) << step_shift));
 }
 
-int64 S2CellId::distance_from_begin() const {
+int64_t S2CellId::distance_from_begin() const {
   const int step_shift = 2 * (kMaxLevel - level()) + 1;
   return id_ >> step_shift;
 }
 
-S2CellId S2CellId::advance_wrap(int64 steps) const {
-  S2_DCHECK(is_valid());
+S2CellId S2CellId::advance_wrap(int64_t steps) const {
+  ABSL_DCHECK(is_valid());
   if (steps == 0) return *this;
 
   int step_shift = 2 * (kMaxLevel - level()) + 1;
   if (steps < 0) {
-    int64 min_steps = -static_cast<int64>(id_ >> step_shift);
+    int64_t min_steps = -static_cast<int64_t>(id_ >> step_shift);
     if (steps < min_steps) {
-      int64 step_wrap = kWrapOffset >> step_shift;
+      int64_t step_wrap = kWrapOffset >> step_shift;
       steps %= step_wrap;
       if (steps < min_steps) steps += step_wrap;
     }
   } else {
     // Unlike advance(), we don't want to return End(level).
-    int64 max_steps = (kWrapOffset - id_) >> step_shift;
+    int64_t max_steps = (kWrapOffset - id_) >> step_shift;
     if (steps > max_steps) {
-      int64 step_wrap = kWrapOffset >> step_shift;
+      int64_t step_wrap = kWrapOffset >> step_shift;
       steps %= step_wrap;
       if (steps > max_steps) steps -= step_wrap;
     }
   }
-  return S2CellId(id_ + (static_cast<uint64>(steps) << step_shift));
+  return S2CellId(id_ + (static_cast<uint64_t>(steps) << step_shift));
 }
 
 S2CellId S2CellId::maximum_tile(const S2CellId limit) const {
@@ -171,7 +176,9 @@ S2CellId S2CellId::maximum_tile(const S2CellId limit) const {
     // of S2CellId ranges, this loop usually executes only once.  Also because
     // id.range_min() < limit.range_min(), we will always exit the loop by the
     // time we reach a leaf cell.
-    do { id = id.child(0); } while (id.range_max() >= limit);
+    do {
+      id = id.child(0);
+    } while (id.range_max() >= limit);
     return id;
   }
   // The cell may be too small.  Grow it if necessary.  Note that generally
@@ -188,17 +195,20 @@ int S2CellId::GetCommonAncestorLevel(S2CellId other) const {
   // Basically we find the first bit position at which the two S2CellIds
   // differ and convert that to a level.  The max() below is necessary for the
   // case where one S2CellId is a descendant of the other.
-  uint64 bits = max(id() ^ other.id(), max(lsb(), other.lsb()));
-  S2_DCHECK_NE(bits, 0);  // Because lsb() is non-zero.
+  uint64_t bits = max(id() ^ other.id(), max(lsb(), other.lsb()));
+  ABSL_ASSUME(bits != 0);  // Because lsb() is non-zero.
 
   // Compute the position of the most significant bit, and then map the bit
   // position as follows:
   // {0} -> 30, {1,2} -> 29, {3,4} -> 28, ... , {59,60} -> 0, {61,62,63} -> -1.
-  return max(60 - Bits::FindMSBSetNonZero64(bits), -1) >> 1;
+  // Work around https://github.com/abseil/abseil-cpp/issues/1890 with
+  // `static_cast`.  This can be removed when we require an abseil-cpp LTS
+  // with the fix, perhaps 2025-07.
+  return max(61 - static_cast<int>(absl::bit_width(bits)), -1) >> 1;
 }
 
 // Print the num_digits low order hex digits.
-static string HexFormatString(uint64 val, size_t num_digits) {
+static string HexFormatString(uint64_t val, size_t num_digits) {
   string result(num_digits, ' ');
   for (; num_digits--; val >>= 4)
     result[num_digits] = "0123456789abcdef"[val & 0xF];
@@ -219,15 +229,17 @@ string S2CellId::ToToken() const {
   // "0" with trailing 0s stripped is the empty string, which is not a
   // reasonable token.  Encode as "X".
   if (id_ == 0) return "X";
-  const size_t num_zero_digits = Bits::FindLSBSetNonZero64(id_) / 4;
+  const size_t num_zero_digits = absl::countr_zero(id_) / 4;
   return HexFormatString(id_ >> (4 * num_zero_digits), 16 - num_zero_digits);
 }
 
-S2CellId S2CellId::FromToken(const absl::string_view token) {
+S2CellId S2CellId::FromToken(const string_view token) {
   if (token.length() > 16) return S2CellId::None();
-  uint64 id = 0;
-  for (int i = 0, pos = 60; i < token.length(); ++i, pos -= 4) {
-    uint64 d;
+  uint64_t id = 0;
+  // Use size_t to fix signed/unsigned comparison for client that use `-Wextra`
+  // (e.g. Chrome).
+  for (size_t i = 0, pos = 60; i < token.length(); ++i, pos -= 4) {
+    uint64_t d;
     if ('0' <= token[i] && token[i] <= '9') {
       d = token[i] - '0';
     } else if ('a' <= token[i] && token[i] <= 'f') {
@@ -243,45 +255,43 @@ S2CellId S2CellId::FromToken(const absl::string_view token) {
 }
 
 void S2CellId::Encode(Encoder* const encoder) const {
-  encoder->Ensure(sizeof(uint64));  // A single uint64.
+  encoder->Ensure(sizeof(uint64_t));  // A single uint64_t.
   encoder->put64(id_);
 }
 
 bool S2CellId::Decode(Decoder* const decoder) {
-  if (decoder->avail() < sizeof(uint64)) return false;
+  if (decoder->avail() < sizeof(uint64_t)) return false;
   id_ = decoder->get64();
   return true;
 }
 
 S2CellId S2CellId::FromFaceIJ(int face, int i, int j) {
-  // Initialization if not done yet
-  MaybeInit();
-
   // Optimization notes:
   //  - Non-overlapping bit fields can be combined with either "+" or "|".
   //    Generally "+" seems to produce better code, but not always.
 
   // Note that this value gets shifted one bit to the left at the end
   // of the function.
-  uint64 n = absl::implicit_cast<uint64>(face) << (kPosBits - 1);
+  uint64_t n = absl::implicit_cast<uint64_t>(face) << (kPosBits - 1);
 
   // Alternating faces have opposite Hilbert curve orientations; this
   // is necessary in order for all faces to have a right-handed
   // coordinate system.
-  uint64 bits = (face & kSwapMask);
+  uint64_t bits = (face & kSwapMask);
 
   // Each iteration maps 4 bits of "i" and "j" into 8 bits of the Hilbert
   // curve position.  The lookup table transforms a 10-bit key of the form
   // "iiiijjjjoo" to a 10-bit value of the form "ppppppppoo", where the
   // letters [ijpo] denote bits of "i", "j", Hilbert curve position, and
   // Hilbert curve orientation respectively.
-#define GET_BITS(k) do { \
-    const int mask = (1 << kLookupBits) - 1; \
+#define GET_BITS(k)                                                 \
+  do {                                                              \
+    const int mask = (1 << kLookupBits) - 1;                        \
     bits += ((i >> (k * kLookupBits)) & mask) << (kLookupBits + 2); \
-    bits += ((j >> (k * kLookupBits)) & mask) << 2; \
-    bits = lookup_pos[bits]; \
-    n |= (bits >> 2) << (k * 2 * kLookupBits); \
-    bits &= (kSwapMask | kInvertMask); \
+    bits += ((j >> (k * kLookupBits)) & mask) << 2;                 \
+    bits = kLookup.pos[bits];                                       \
+    n |= (bits >> 2) << (k * 2 * kLookupBits);                      \
+    bits &= (kSwapMask | kInvertMask);                              \
   } while (0)
 
   GET_BITS(7);
@@ -305,14 +315,9 @@ S2CellId::S2CellId(const S2Point& p) {
   id_ = FromFaceIJ(face, i, j).id();
 }
 
-S2CellId::S2CellId(const S2LatLng& ll)
-  : S2CellId(ll.ToPoint()) {
-}
+S2CellId::S2CellId(const S2LatLng& ll) : S2CellId(ll.ToPoint()) {}
 
 int S2CellId::ToFaceIJOrientation(int* pi, int* pj, int* orientation) const {
-  // Initialization if not done yet
-  MaybeInit();
-
   int i = 0, j = 0;
   int face = this->face();
   int bits = (face & kSwapMask);
@@ -325,14 +330,16 @@ int S2CellId::ToFaceIJOrientation(int* pi, int* pj, int* orientation) const {
   //
   // On the first iteration we need to be careful to clear out the bits
   // representing the cube face.
-#define GET_BITS(k) do { \
+#define GET_BITS(k)                                                           \
+  do {                                                                        \
     const int nbits = (k == 7) ? (kMaxLevel - 7 * kLookupBits) : kLookupBits; \
-    bits += (static_cast<int>(id_ >> (k * 2 * kLookupBits + 1)) \
-             & ((1 << (2 * nbits)) - 1)) << 2; \
-    bits = lookup_ij[bits]; \
-    i += (bits >> (kLookupBits + 2)) << (k * kLookupBits); \
-    j += ((bits >> 2) & ((1 << kLookupBits) - 1)) << (k * kLookupBits); \
-    bits &= (kSwapMask | kInvertMask); \
+    bits += (static_cast<int>(id_ >> (k * 2 * kLookupBits + 1)) &             \
+             ((1 << (2 * nbits)) - 1))                                        \
+            << 2;                                                             \
+    bits = kLookup.ij[bits];                                                  \
+    i += (bits >> (kLookupBits + 2)) << (k * kLookupBits);                    \
+    j += ((bits >> 2) & ((1 << kLookupBits) - 1)) << (k * kLookupBits);       \
+    bits &= (kSwapMask | kInvertMask);                                        \
   } while (0)
 
   GET_BITS(7);
@@ -356,8 +363,8 @@ int S2CellId::ToFaceIJOrientation(int* pi, int* pj, int* orientation) const {
     // by (kMaxLevel-n-1) repetitions of "00", followed by "0".  The "10" has
     // no effect, while each occurrence of "00" has the effect of reversing
     // the kSwapMask bit.
-    S2_DCHECK_EQ(0, kPosToOrientation[2]);
-    S2_DCHECK_EQ(kSwapMask, kPosToOrientation[0]);
+    ABSL_DCHECK_EQ(0, kPosToOrientation[2]);
+    ABSL_DCHECK_EQ(kSwapMask, kPosToOrientation[0]);
     if (lsb() & 0x1111111111111110ULL) {
       bits ^= kSwapMask;
     }
@@ -372,9 +379,7 @@ S2Point S2CellId::ToPointRaw() const {
   return S2::FaceSiTitoXYZ(face, si, ti);
 }
 
-S2LatLng S2CellId::ToLatLng() const {
-  return S2LatLng(ToPointRaw());
-}
+S2LatLng S2CellId::ToLatLng() const { return S2LatLng(ToPointRaw()); }
 
 R2Point S2CellId::GetCenterST() const {
   int si, ti;
@@ -385,8 +390,7 @@ R2Point S2CellId::GetCenterST() const {
 R2Point S2CellId::GetCenterUV() const {
   int si, ti;
   GetCenterSiTi(&si, &ti);
-  return R2Point(S2::STtoUV(S2::SiTitoST(si)),
-                 S2::STtoUV(S2::SiTitoST(ti)));
+  return R2Point(S2::STtoUV(S2::SiTitoST(si)), S2::STtoUV(S2::SiTitoST(ti)));
 }
 
 R2Rect S2CellId::IJLevelToBoundUV(int ij[2], int level) {
@@ -421,8 +425,8 @@ R2Rect S2CellId::GetBoundUV() const {
 static double ExpandEndpoint(double u, double max_v, double sin_dist) {
   // This is based on solving a spherical right triangle, similar to the
   // calculation in S2Cap::GetRectBound.
-  double sin_u_shift = sin_dist * sqrt((1 + u * u + max_v * max_v) /
-                                       (1 + u * u));
+  double sin_u_shift =
+      sin_dist * sqrt((1 + u * u + max_v * max_v) / (1 + u * u));
   double cos_u_shift = sqrt(1 - sin_u_shift * sin_u_shift);
   // The following is an expansion of tan(atan(u) + asin(sin_u_shift)).
   return (cos_u_shift * u + sin_u_shift) / (cos_u_shift - sin_u_shift * u);
@@ -437,10 +441,19 @@ R2Rect S2CellId::ExpandedByDistanceUV(const R2Rect& uv, S1Angle distance) {
   double max_u = max(fabs(u0), fabs(u1));
   double max_v = max(fabs(v0), fabs(v1));
   double sin_dist = sin(distance);
-  return R2Rect(R1Interval(ExpandEndpoint(u0, max_v, -sin_dist),
-                           ExpandEndpoint(u1, max_v, sin_dist)),
-                R1Interval(ExpandEndpoint(v0, max_u, -sin_dist),
-                           ExpandEndpoint(v1, max_u, sin_dist)));
+
+  R1Interval xinterv = R1Interval(ExpandEndpoint(u0, max_v, -sin_dist),
+                                  ExpandEndpoint(u1, max_v, sin_dist));
+  R1Interval yinterv = R1Interval(ExpandEndpoint(v0, max_u, -sin_dist),
+                                  ExpandEndpoint(v1, max_u, sin_dist));
+
+  // R2Rect requires both or neither dimension be empty, so if we shrank the
+  // rectangle too much, manually collapse to a degenerate rectangle at the
+  // first corner.
+  if (xinterv.is_empty() || yinterv.is_empty()) {
+    return R2Rect(R1Interval(u0, u0), R1Interval(v0, v0));
+  }
+  return R2Rect(xinterv, yinterv);
 }
 
 S2CellId S2CellId::FromFaceIJWrap(int face, int i, int j) {
@@ -466,14 +479,14 @@ S2CellId S2CellId::FromFaceIJWrap(int face, int i, int j) {
   static const double kScale = 1.0 / kMaxSize;
   static const double kLimit = 1.0 + DBL_EPSILON;
   // The arithmetic below is designed to avoid 32-bit integer overflows.
-  S2_DCHECK_EQ(0, kMaxSize % 2);
+  ABSL_DCHECK_EQ(0, kMaxSize % 2);
   double u = max(-kLimit, min(kLimit, kScale * (2 * (i - kMaxSize / 2) + 1)));
   double v = max(-kLimit, min(kLimit, kScale * (2 * (j - kMaxSize / 2) + 1)));
 
   // Find the leaf cell coordinates on the adjacent face, and convert
   // them to a cell id at the appropriate level.
   face = S2::XYZtoFaceUV(S2::FaceUVtoXYZ(face, u, v), &u, &v);
-  return FromFaceIJ(face, S2::STtoIJ(0.5*(u+1)), S2::STtoIJ(0.5*(v+1)));
+  return FromFaceIJ(face, S2::STtoIJ(0.5 * (u + 1)), S2::STtoIJ(0.5 * (v + 1)));
 }
 
 inline S2CellId S2CellId::FromFaceIJSame(int face, int i, int j,
@@ -491,21 +504,19 @@ void S2CellId::GetEdgeNeighbors(S2CellId neighbors[4]) const {
   int face = ToFaceIJOrientation(&i, &j, nullptr);
 
   // Edges 0, 1, 2, 3 are in the down, right, up, left directions.
-  neighbors[0] = FromFaceIJSame(face, i, j - size, j - size >= 0)
-                 .parent(level);
-  neighbors[1] = FromFaceIJSame(face, i + size, j, i + size < kMaxSize)
-                 .parent(level);
-  neighbors[2] = FromFaceIJSame(face, i, j + size, j + size < kMaxSize)
-                 .parent(level);
-  neighbors[3] = FromFaceIJSame(face, i - size, j, i - size >= 0)
-                 .parent(level);
+  neighbors[0] = FromFaceIJSame(face, i, j - size, j - size >= 0).parent(level);
+  neighbors[1] =
+      FromFaceIJSame(face, i + size, j, i + size < kMaxSize).parent(level);
+  neighbors[2] =
+      FromFaceIJSame(face, i, j + size, j + size < kMaxSize).parent(level);
+  neighbors[3] = FromFaceIJSame(face, i - size, j, i - size >= 0).parent(level);
 }
 
 void S2CellId::AppendVertexNeighbors(int level,
                                      vector<S2CellId>* output) const {
   // "level" must be strictly less than this cell's level so that we can
   // determine which vertex this cell is closest to.
-  S2_DCHECK_LT(level, this->level());
+  ABSL_DCHECK_LT(level, this->level());
   int i, j;
   int face = ToFaceIJOrientation(&i, &j, nullptr);
 
@@ -537,14 +548,15 @@ void S2CellId::AppendVertexNeighbors(int level,
   // If i- and j- edge neighbors are *both* on a different face, then this
   // vertex only has three neighbors (it is one of the 8 cube vertices).
   if (isame || jsame) {
-    output->push_back(FromFaceIJSame(face, i + ioffset, j + joffset,
-                                     isame && jsame).parent(level));
+    output->push_back(
+        FromFaceIJSame(face, i + ioffset, j + joffset, isame && jsame)
+            .parent(level));
   }
 }
 
 void S2CellId::AppendAllNeighbors(int nbr_level,
                                   vector<S2CellId>* output) const {
-  S2_DCHECK_GE(nbr_level, level());
+  ABSL_DCHECK_GE(nbr_level, level());
   int i, j;
   int face = ToFaceIJOrientation(&i, &j, nullptr);
 
@@ -556,11 +568,11 @@ void S2CellId::AppendAllNeighbors(int nbr_level,
   j &= -size;
 
   int nbr_size = GetSizeIJ(nbr_level);
-  S2_DCHECK_LE(nbr_size, size);
+  ABSL_DCHECK_LE(nbr_size, size);
 
   // We compute the top-bottom, left-right, and diagonal neighbors in one
   // pass.  The loop test is at the end of the loop to avoid 32-bit overflow.
-  for (int k = -nbr_size; ; k += nbr_size) {
+  for (int k = -nbr_size;; k += nbr_size) {
     bool same_face;
     if (k < 0) {
       same_face = (j + k >= 0);
@@ -569,18 +581,19 @@ void S2CellId::AppendAllNeighbors(int nbr_level,
     } else {
       same_face = true;
       // Top and bottom neighbors.
-      output->push_back(FromFaceIJSame(face, i + k, j - nbr_size,
-                                       j - size >= 0).parent(nbr_level));
-      output->push_back(FromFaceIJSame(face, i + k, j + size,
-                                       j + size < kMaxSize).parent(nbr_level));
+      output->push_back(FromFaceIJSame(face, i + k, j - nbr_size, j - size >= 0)
+                            .parent(nbr_level));
+      output->push_back(
+          FromFaceIJSame(face, i + k, j + size, j + size < kMaxSize)
+              .parent(nbr_level));
     }
     // Left, right, and diagonal neighbors.
-    output->push_back(FromFaceIJSame(face, i - nbr_size, j + k,
-                                     same_face && i - size >= 0)
-                      .parent(nbr_level));
-    output->push_back(FromFaceIJSame(face, i + size, j + k,
-                                     same_face && i + size < kMaxSize)
-                      .parent(nbr_level));
+    output->push_back(
+        FromFaceIJSame(face, i - nbr_size, j + k, same_face && i - size >= 0)
+            .parent(nbr_level));
+    output->push_back(
+        FromFaceIJSame(face, i + size, j + k, same_face && i + size < kMaxSize)
+            .parent(nbr_level));
     if (k >= size) break;
   }
 }
@@ -602,7 +615,7 @@ std::ostream& operator<<(std::ostream& os, S2CellId id) {
   return os << id.ToString();
 }
 
-S2CellId S2CellId::FromDebugString(absl::string_view str) {
+S2CellId S2CellId::FromDebugString(string_view str) {
   // This function is reasonably efficient, but is only intended for use in
   // tests.
   int level = static_cast<int>(str.size() - 2);
@@ -610,10 +623,58 @@ S2CellId S2CellId::FromDebugString(absl::string_view str) {
   int face = str[0] - '0';
   if (face < 0 || face > 5 || str[1] != '/') return S2CellId::None();
   S2CellId id = S2CellId::FromFace(face);
-  for (int i = 2; i < str.size(); ++i) {
+  // Use size_t to fix signed/unsigned comparison for client that use `-Wextra`
+  // (e.g. Chrome).
+  for (size_t i = 2; i < str.size(); ++i) {
     int child_pos = str[i] - '0';
     if (child_pos < 0 || child_pos > 3) return S2CellId::None();
     id = id.child(child_pos);
   }
   return id;
 }
+
+void S2CellId::Coder::Encode(Encoder& encoder, const S2CellId& v) const {
+  string token = v.ToToken();
+  // Ensure enough space for the token plus 1 nul byte appended by
+  // Encoder::puts.
+  encoder.Ensure(token.length() + 1);
+  encoder.puts(token.c_str());
+}
+
+bool S2CellId::Coder::Decode(Decoder& decoder, S2CellId& v,
+                             S2Error& error) const {
+  // The longest S2CellId representation is 16 bytes, plus one more for the nul
+  // terminator.
+  char bytes[17];
+
+  const size_t start_pos = decoder.pos();
+  decoder.getcn(bytes, '\0', std::min(decoder.avail(), sizeof(bytes)));
+  const size_t bytes_read = decoder.pos() - start_pos;
+
+  // The token must be nul-terminated.
+  if (bytes_read == 0 || bytes[bytes_read - 1] != '\0') {
+    error = S2Error::DataLoss("Unknown decoding error");
+    return false;
+  }
+
+  const string_view token(bytes, bytes_read - 1);
+  v = S2CellId::FromToken(token);
+  // Prevent edge cases where S2CellId::FromToken returns S2CellId::None for
+  // an invalid token.
+  if (v == S2CellId::None() && token != "X") {
+    error = S2Error::DataLoss("Unknown decoding error");
+    return false;
+  }
+  return true;
+}
+
+bool AbslParseFlag(string_view input, S2CellId* id, string* error) {
+  *id = S2CellId::FromToken(input);
+  if (!id->is_valid()) {
+    *error = absl::StrCat("Error. Expected valid S2 token got: '", input, "'");
+    return false;
+  }
+  return true;
+}
+
+string AbslUnparseFlag(S2CellId id) { return id.ToToken(); }
