@@ -19,13 +19,15 @@
 #define S2_S2CELL_INDEX_H_
 
 #include <algorithm>
+#include <cstdint>
 #include <functional>
+#include <ostream>
 #include <vector>
 
 #include "absl/container/flat_hash_set.h"
+#include "absl/log/absl_check.h"
 
-#include "s2/base/integral_types.h"
-#include "s2/base/logging.h"
+#include "s2/_fp_contract_off.h"  // IWYU pragma: keep
 #include "s2/base/log_severity.h"
 #include "s2/s2cell_id.h"
 #include "s2/s2cell_union.h"
@@ -45,7 +47,7 @@
 //   string label_str = ...;
 //   cell_index.Add(cell_id, my_label_lexicon.Add(label_str));
 //   ...
-//   int32 label = ...;
+//   int32_t label = ...;
 //   string label_str = my_label_lexicon.value(label);
 //
 // To build an S2CellIndex, call Add() for each (cell_id, label) pair, and
@@ -110,17 +112,12 @@ class S2CellIndex {
   //
   //   ValueLexicon<MyLabel> my_label_lexicon;
   //   index.Add(cell_id, my_label_lexicon.Add(label));
-  using Label = int32;
+  using Label = int32_t;
 
   // Convenience class that represents a (cell_id, label) pair.
   struct LabelledCell {
-    S2CellId cell_id;
-    Label label;
-
-    LabelledCell() : cell_id(S2CellId::None()), label(-1) {}
-
-    LabelledCell(S2CellId _cell_id, Label _label)
-        : cell_id(_cell_id), label(_label) {}
+    S2CellId cell_id = S2CellId::None();
+    Label label = -1;
 
     bool operator==(LabelledCell y) const {
       return cell_id == y.cell_id && label == y.label;
@@ -131,10 +128,12 @@ class S2CellIndex {
       if (y.cell_id < cell_id) return false;
       return label < y.label;
     }
-  };
 
-  // Default constructor.
-  S2CellIndex();
+    template <typename H>
+    friend H AbslHashValue(H h, LabelledCell x) {
+      return H::combine(std::move(h), x.cell_id, x.label);
+    }
+  };
 
   // Returns the number of (cell_id, label) pairs in the index.
   int num_cells() const;
@@ -192,14 +191,9 @@ class S2CellIndex {
   // Represents a node in the (cell_id, label) tree.  Cells are organized in a
   // tree such that the ancestors of a given node contain that node.
   struct CellNode {
-    S2CellId cell_id;
-    Label label;
-    int32 parent;
-
-    CellNode(S2CellId _cell_id, Label _label, int32 _parent)
-        : cell_id(_cell_id), label(_label), parent(_parent) {
-    }
-    CellNode() : cell_id(S2CellId::None()), label(kDoneContents), parent(-1) {}
+    S2CellId cell_id = S2CellId::None();
+    Label label = kDoneContents;
+    int32_t parent = -1;
   };
 
  public:
@@ -388,7 +382,7 @@ class S2CellIndex {
     void set_done() { node_.label = kDoneContents; }
 
     // A pointer to the cell tree itself (owned by the S2CellIndex).
-    const std::vector<CellNode>* cell_tree_;
+    const std::vector<CellNode>* cell_tree_ = nullptr;
 
     // The value of it.start_id() from the previous call to StartUnion().
     // This is used to check whether these values are monotonically
@@ -398,11 +392,11 @@ class S2CellIndex {
     // The maximum index within the cell_tree_ vector visited during the
     // previous call to StartUnion().  This is used to eliminate duplicate
     // values when StartUnion() is called multiple times.
-    int32 node_cutoff_;
+    int32_t node_cutoff_;
 
     // The maximum index within the cell_tree_ vector visited during the
     // current call to StartUnion().  This is used to update node_cutoff_.
-    int32 next_node_cutoff_;
+    int32_t next_node_cutoff_;
 
     // A copy of the current node in the cell tree.
     CellNode node_;
@@ -424,11 +418,10 @@ class S2CellIndex {
   // cells that overlap this range.
   struct RangeNode {
     S2CellId start_id;  // First leaf cell contained by this range.
-    int32 contents;     // Contents of this node (an index within cell_tree_).
+    int32_t contents;   // Contents of this node (an index within cell_tree_).
 
-    RangeNode(S2CellId _start_id, int32 _contents)
-        : start_id(_start_id), contents(_contents) {
-    }
+    RangeNode(S2CellId _start_id, int32_t _contents)
+        : start_id(_start_id), contents(_contents) {}
 
     // Comparison operator needed for std::upper_bound().
     friend bool operator<(S2CellId x, const RangeNode& y) {
@@ -438,9 +431,6 @@ class S2CellIndex {
   // The last element of range_nodes_ is a sentinel value, which is necessary
   // in order to represent the range covered by the previous element.
   std::vector<RangeNode> range_nodes_;
-
-  S2CellIndex(const S2CellIndex&) = delete;
-  void operator=(const S2CellIndex&) = delete;
 };
 std::ostream& operator<<(std::ostream& os, S2CellIndex::LabelledCell x);
 
@@ -450,23 +440,23 @@ std::ostream& operator<<(std::ostream& os, S2CellIndex::LabelledCell x);
 inline S2CellIndex::CellIterator::CellIterator(const S2CellIndex* index)
     : cell_it_(index->cell_tree_.begin()),
       cell_end_(index->cell_tree_.end()) {
-  S2_DCHECK(!index->range_nodes_.empty()) << "Call Build() first.";
+  ABSL_DCHECK(!index->range_nodes_.empty()) << "Call Build() first.";
 }
 
 inline S2CellId S2CellIndex::CellIterator::cell_id() const {
-  S2_DCHECK(!done());
+  ABSL_DCHECK(!done());
   return cell_it_->cell_id;
 }
 
 inline S2CellIndex::Label S2CellIndex::CellIterator::label() const {
-  S2_DCHECK(!done());
+  ABSL_DCHECK(!done());
   return cell_it_->label;
 }
 
 inline S2CellIndex::LabelledCell S2CellIndex::CellIterator::labelled_cell()
     const {
-  S2_DCHECK(!done());
-  return LabelledCell(cell_it_->cell_id, cell_it_->label);
+  ABSL_DCHECK(!done());
+  return LabelledCell{cell_it_->cell_id, cell_it_->label};
 }
 
 inline bool S2CellIndex::CellIterator::done() const {
@@ -474,14 +464,14 @@ inline bool S2CellIndex::CellIterator::done() const {
 }
 
 inline void S2CellIndex::CellIterator::Next() {
-  S2_DCHECK(!done());
+  ABSL_DCHECK(!done());
   ++cell_it_;
 }
 
 inline S2CellIndex::RangeIterator::RangeIterator(const S2CellIndex* index)
     : range_nodes_(&index->range_nodes_), it_() {
-  S2_DCHECK(!range_nodes_->empty()) << "Call Build() first.";
-  if (google::DEBUG_MODE) it_ = kUninitialized();  // See done().
+  ABSL_DCHECK(!range_nodes_->empty()) << "Call Build() first.";
+  if (S2_DEBUG_MODE) it_ = kUninitialized();  // See done().
 }
 
 inline S2CellId S2CellIndex::RangeIterator::start_id() const {
@@ -489,12 +479,12 @@ inline S2CellId S2CellIndex::RangeIterator::start_id() const {
 }
 
 inline S2CellId S2CellIndex::RangeIterator::limit_id() const {
-  S2_DCHECK(!done());
+  ABSL_DCHECK(!done());
   return (it_ + 1)->start_id;
 }
 
 inline bool S2CellIndex::RangeIterator::done() const {
-  S2_DCHECK(it_ != kUninitialized()) << "Call Begin() or Seek() first.";
+  ABSL_DCHECK(it_ != kUninitialized()) << "Call Begin() or Seek() first.";
 
   // Note that the last element of range_nodes_ is a sentinel value.
   return it_ >= range_nodes_->end() - 1;
@@ -510,7 +500,7 @@ inline void S2CellIndex::RangeIterator::Finish() {
 }
 
 inline void S2CellIndex::RangeIterator::Next() {
-  S2_DCHECK(!done());
+  ABSL_DCHECK(!done());
   ++it_;
 }
 
@@ -561,9 +551,7 @@ inline bool S2CellIndex::RangeIterator::Prev() {
   return true;
 }
 
-inline S2CellIndex::ContentsIterator::ContentsIterator()
-    : cell_tree_(nullptr) {
-}
+inline S2CellIndex::ContentsIterator::ContentsIterator() = default;
 
 inline S2CellIndex::ContentsIterator::ContentsIterator(
     const S2CellIndex* index) {
@@ -583,19 +571,19 @@ inline void S2CellIndex::ContentsIterator::Clear() {
 }
 
 inline S2CellId S2CellIndex::ContentsIterator::cell_id() const {
-  S2_DCHECK(!done());
+  ABSL_DCHECK(!done());
   return node_.cell_id;
 }
 
 inline S2CellIndex::Label S2CellIndex::ContentsIterator::label() const {
-  S2_DCHECK(!done());
+  ABSL_DCHECK(!done());
   return node_.label;
 }
 
 inline S2CellIndex::LabelledCell S2CellIndex::ContentsIterator::labelled_cell()
     const {
-  S2_DCHECK(!done());
-  return LabelledCell(node_.cell_id, node_.label);
+  ABSL_DCHECK(!done());
+  return LabelledCell{node_.cell_id, node_.label};
 }
 
 inline bool S2CellIndex::ContentsIterator::done() const {
@@ -603,7 +591,7 @@ inline bool S2CellIndex::ContentsIterator::done() const {
 }
 
 inline void S2CellIndex::ContentsIterator::Next() {
-  S2_DCHECK(!done());
+  ABSL_DCHECK(!done());
   if (node_.parent <= node_cutoff_) {
     // We have already processed this node and its ancestors.
     node_cutoff_ = next_node_cutoff_;
@@ -618,9 +606,9 @@ inline int S2CellIndex::num_cells() const {
 }
 
 inline void S2CellIndex::Add(S2CellId cell_id, Label label) {
-  S2_DCHECK(cell_id.is_valid());
-  S2_DCHECK_GE(label, 0);
-  cell_tree_.push_back(CellNode(cell_id, label, -1));
+  ABSL_DCHECK(cell_id.is_valid());
+  ABSL_DCHECK_GE(label, 0);
+  cell_tree_.push_back(CellNode{cell_id, label, -1});
 }
 
 inline void S2CellIndex::Clear() {
