@@ -20,28 +20,32 @@
 #include <algorithm>
 #include <cfloat>
 #include <cmath>
-#include <cstdio>
-#include <map>
+#include <cstdint>
+#include <string>
 #include <utility>
 #include <vector>
 
+#include <gmock/gmock.h>
 #include <gtest/gtest.h>
 
-#include "absl/base/macros.h"
+#include "absl/container/flat_hash_map.h"
 #include "absl/flags/flag.h"
-#include "absl/memory/memory.h"
+#include "absl/log/absl_check.h"
+#include "absl/log/log_streamer.h"
+#include "absl/random/bit_gen_ref.h"
+#include "absl/random/random.h"
 #include "absl/strings/str_cat.h"
+#include "absl/strings/str_format.h"
+#include "absl/strings/string_view.h"
 
-#include "s2/base/logging.h"
 #include "s2/base/log_severity.h"
-#include "s2/mutable_s2shape_index.h"
 #include "s2/r2.h"
 #include "s2/r2rect.h"
 #include "s2/s1angle.h"
 #include "s2/s1chord_angle.h"
 #include "s2/s1interval.h"
 #include "s2/s2cap.h"
-#include "s2/s2crossing_edge_query.h"
+#include "s2/s2cell_id.h"
 #include "s2/s2edge_crossings.h"
 #include "s2/s2edge_distances.h"
 #include "s2/s2latlng.h"
@@ -49,23 +53,28 @@
 #include "s2/s2latlng_rect_bounder.h"
 #include "s2/s2loop.h"
 #include "s2/s2metrics.h"
+#include "s2/s2point.h"
 #include "s2/s2pointutil.h"
+#include "s2/s2random.h"
 #include "s2/s2testing.h"
 #include "s2/s2text_format.h"
+#include "s2/util/coding/coder.h"
 
+using absl::flat_hash_map;
 using absl::StrCat;
+using absl::string_view;
 using S2::internal::kSwapMask;
 using s2textformat::MakePointOrDie;
 using std::fabs;
-using std::map;
 using std::max;
 using std::min;
 using std::pow;
 using std::vector;
+using ::testing::Eq;
 
 TEST(S2Cell, TestFaces) {
-  map<S2Point, int> edge_counts;
-  map<S2Point, int> vertex_counts;
+  flat_hash_map<S2Point, int> edge_counts;
+  flat_hash_map<S2Point, int> vertex_counts;
   for (int face = 0; face < 6; ++face) {
     S2CellId id = S2CellId::FromFace(face);
     S2Cell cell(id);
@@ -162,12 +171,12 @@ static void GatherStats(const S2Cell& cell) {
   s->max_approx_ratio = max(approx_ratio, s->max_approx_ratio);
 }
 
-static void TestSubdivide(const S2Cell& cell) {
+static void TestSubdivide(absl::BitGenRef bitgen, const S2Cell& cell) {
   GatherStats(cell);
   if (cell.is_leaf()) return;
 
   S2Cell children[4];
-  S2_CHECK(cell.Subdivide(children));
+  ABSL_CHECK(cell.Subdivide(children));
   S2CellId child_id = cell.id().child_begin();
   double exact_area = 0;
   double approx_area = 0;
@@ -261,10 +270,9 @@ static void TestSubdivide(const S2Cell& cell) {
       if (children[i].GetBoundUV().Contains(uv))
         force_subdivide = true;
     }
-    if (force_subdivide ||
-        cell.level() < (google::DEBUG_MODE ? 5 : 6) ||
-        S2Testing::rnd.OneIn(google::DEBUG_MODE ? 5 : 4)) {
-      TestSubdivide(children[i]);
+    if (force_subdivide || cell.level() < (S2_DEBUG_MODE ? 5 : 6) ||
+        absl::Bernoulli(bitgen, S2_DEBUG_MODE ? 0.2 : 0.25)) {
+      TestSubdivide(bitgen, children[i]);
     }
   }
 
@@ -285,13 +293,11 @@ static void TestSubdivide(const S2Cell& cell) {
 }
 
 template <int dim>
-static void CheckMinMaxAvg(
-    const char* label, int level, double count, double abs_error,
-    double min_value, double max_value, double avg_value,
-    const S2::Metric<dim>& min_metric,
-    const S2::Metric<dim>& max_metric,
-    const S2::Metric<dim>& avg_metric) {
-
+static void CheckMinMaxAvg(string_view label, int level, double count,
+                           double abs_error, double min_value, double max_value,
+                           double avg_value, const S2::Metric<dim>& min_metric,
+                           const S2::Metric<dim>& max_metric,
+                           const S2::Metric<dim>& avg_metric) {
   // All metrics are minimums, maximums, or averages of differential
   // quantities, and therefore will not be exact for cells at any finite
   // level.  The differential minimum is always a lower bound, and the maximum
@@ -313,12 +319,13 @@ static void CheckMinMaxAvg(
   double min_error = min_value - min_metric.GetValue(level);
   double max_error = max_metric.GetValue(level) - max_value;
   double avg_error = fabs(avg_metric.GetValue(level) - avg_value);
-  printf("%-10s (%6.0f samples, tolerance %8.3g) - min %9.4g (%9.3g : %9.3g) "
-         "max %9.4g (%9.3g : %9.3g), avg %9.4g (%9.3g : %9.3g)\n",
-         label, count, tolerance,
-         min_value, min_error / min_value, min_error / tolerance,
-         max_value, max_error / max_value, max_error / tolerance,
-         avg_value, avg_error / avg_value, avg_error / tolerance);
+  absl::PrintF(
+      "%-10s (%6.0f samples, tolerance %8.3g) - min %9.4g (%9.3g : %9.3g) "
+      "max %9.4g (%9.3g : %9.3g), avg %9.4g (%9.3g : %9.3g)\n",
+      label, count, tolerance, min_value, min_error / min_value,
+      min_error / tolerance, max_value, max_error / max_value,
+      max_error / tolerance, avg_value, avg_error / avg_value,
+      avg_error / tolerance);
 
   EXPECT_LE(min_metric.GetValue(level), min_value + abs_error);
   EXPECT_GE(min_metric.GetValue(level), min_value - tolerance);
@@ -328,10 +335,13 @@ static void CheckMinMaxAvg(
 }
 
 TEST(S2Cell, TestSubdivide) {
+  absl::BitGen bitgen(S2Testing::MakeTaggedSeedSeq(
+      "TEST_SUBDIVIDE", absl::LogInfoStreamer(__FILE__, __LINE__).stream()));
+
   // Only test a sample of faces to reduce the runtime.
-  TestSubdivide(S2Cell::FromFace(0));
-  TestSubdivide(S2Cell::FromFace(3));
-  TestSubdivide(S2Cell::FromFace(5));
+  TestSubdivide(bitgen, S2Cell::FromFace(0));
+  TestSubdivide(bitgen, S2Cell::FromFace(3));
+  TestSubdivide(bitgen, S2Cell::FromFace(5));
 
   // This table is useful in evaluating the quality of the various S2
   // projections.
@@ -343,7 +353,7 @@ TEST(S2Cell, TestSubdivide) {
   // The maximum edge *aspect* is the maximum ratio of the longest edge of a
   // cell to the shortest edge of that same cell (and similarly for the
   // maximum diagonal aspect).
-  printf(
+  absl::PrintF(
       "Ratio:  (Max value for any cell) / (Min value for any cell)\n"
       "Aspect: (Max value / min value) for any cell\n"
       "                   Edge          Diag       Approx Area/    Avg Area/\n"
@@ -359,13 +369,12 @@ TEST(S2Cell, TestSubdivide) {
       s->avg_diag /= s->count;
       s->avg_angle_span /= s->count;
     }
-    printf("%5d  %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f\n",
-           i, s->max_area / s->min_area,
-           s->max_edge / s->min_edge, s->max_edge_aspect,
-           s->max_diag / s->min_diag, s->max_diag_aspect,
-           s->min_approx_ratio, s->max_approx_ratio,
-           S2Cell::AverageArea(i) / s->max_area,
-           S2Cell::AverageArea(i) / s->min_area);
+    absl::PrintF("%5d  %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f\n",
+                 i, s->max_area / s->min_area, s->max_edge / s->min_edge,
+                 s->max_edge_aspect, s->max_diag / s->min_diag,
+                 s->max_diag_aspect, s->min_approx_ratio, s->max_approx_ratio,
+                 S2Cell::AverageArea(i) / s->max_area,
+                 S2Cell::AverageArea(i) / s->min_area);
   }
 
   // Now check the validity of the S2 length and area metrics.
@@ -373,7 +382,8 @@ TEST(S2Cell, TestSubdivide) {
     const LevelStats* s = &level_stats[i];
     if (s->count == 0) continue;
 
-    printf("Level %2d - metric value (error/actual : error/tolerance)\n", i);
+    absl::PrintF("Level %2d - metric value (error/actual : error/tolerance)\n",
+                 i);
 
     // The various length calculations are only accurate to 1e-15 or so,
     // so we need to allow for this amount of discrepancy with the theoretical
@@ -420,8 +430,10 @@ TEST(S2Cell, CellVsLoopRectBound) {
   // Possible additional S2Loop error compared to S2Cell error:
   static S2LatLng kLoopError = S2LatLngRectBounder::MaxErrorForTests();
 
+  absl::BitGen bitgen(S2Testing::MakeTaggedSeedSeq(
+      "CELL_VS_LOOP_RECT_BOUND", absl::LogInfoStreamer(__FILE__, __LINE__).stream()));
   for (int iter = 0; iter < 1000; ++iter) {
-    S2Cell cell(S2Testing::GetRandomCellId());
+    S2Cell cell(s2random::CellId(bitgen));
     S2Loop loop(cell);
     S2LatLngRect cell_bound = cell.GetRectBound();
     S2LatLngRect loop_bound = loop.GetRectBound();
@@ -433,14 +445,16 @@ TEST(S2Cell, CellVsLoopRectBound) {
 TEST(S2Cell, RectBoundIsLargeEnough) {
   // Construct many points that are nearly on an S2Cell edge, and verify that
   // whenever the cell contains a point P then its bound contains S2LatLng(P).
+  absl::BitGen bitgen(S2Testing::MakeTaggedSeedSeq(
+      "RECT_BOUND_IS_LARGE_ENOUGH", absl::LogInfoStreamer(__FILE__, __LINE__).stream()));
 
   for (int iter = 0; iter < 1000; /* advanced in loop below */) {
-    S2Cell cell(S2Testing::GetRandomCellId());
-    int i = S2Testing::rnd.Uniform(4);
+    S2Cell cell(s2random::CellId(bitgen));
+    int i = absl::Uniform(bitgen, 0, 4);
     S2Point v1 = cell.GetVertex(i);
-    S2Point v2 = S2Testing::SamplePoint(
-        S2Cap(cell.GetVertex(i + 1), S1Angle::Radians(1e-15)));
-    S2Point p = S2::Interpolate(v1, v2, S2Testing::rnd.RandDouble());
+    S2Point v2 = s2random::SamplePoint(
+        bitgen, S2Cap(cell.GetVertex(i + 1), S1Angle::Radians(1e-15)));
+    S2Point p = S2::Interpolate(v1, v2, absl::Uniform(bitgen, 0.0, 1.0));
     if (S2Loop(cell).Contains(p)) {
       EXPECT_TRUE(cell.GetRectBound().Contains(S2LatLng(p)));
       ++iter;
@@ -451,18 +465,38 @@ TEST(S2Cell, RectBoundIsLargeEnough) {
 TEST(S2Cell, ConsistentWithS2CellIdFromPoint) {
   // Construct many points that are nearly on an S2Cell edge, and verify that
   // S2Cell(S2CellId(p)).Contains(p) is always true.
+  //
+  // Note this is not actually the case and this test fails 3.5% of the time.
+  // TODO(b/416457492): Fix (S2Cell(S2CellId(point)) does not contain point).
+  absl::BitGen bitgen(
+      S2Testing::MakeTaggedSeedSeq("CONSISTENT_WITH_S2CELL_ID_FROM_POINT",
+                              absl::LogInfoStreamer(__FILE__, __LINE__).stream()));
 
   for (int iter = 0; iter < 1000; ++iter) {
-    S2Cell cell(S2Testing::GetRandomCellId());
-    int i = S2Testing::rnd.Uniform(4);
+    // Leaf cells are better at generating difficult cases near a boundary,
+    // so prefer them.
+    int level =
+        absl::Bernoulli(bitgen, 0.5) ? 30 : absl::Uniform(bitgen, 0, 30);
+    S2Cell cell(s2random::CellId(bitgen, level));
+    int i = absl::Uniform(bitgen, 0, 4);
     S2Point v1 = cell.GetVertex(i);
-    S2Point v2 = S2Testing::SamplePoint(
-        S2Cap(cell.GetVertex(i + 1), S1Angle::Radians(1e-15)));
-    S2Point p = S2::Interpolate(v1, v2, S2Testing::rnd.RandDouble());
-    EXPECT_TRUE(S2Cell(S2CellId(p)).Contains(p));
+    S2Point v2 = s2random::SamplePoint(
+        bitgen, S2Cap(cell.GetVertex(i + 1), S1Angle::Radians(1e-15)));
+    S2Point p = S2::Interpolate(v1, v2, absl::Uniform(bitgen, 0.0, 1.0));
+    EXPECT_TRUE(S2Cell(S2CellId(p)).Contains(p))
+        << " original cell: " << cell.id() << " level: " << cell.level()
+        << " point: " << p << " cell: " << S2CellId(p);
   }
 }
 
+TEST(S2Cell, DISABLED_ConsistentWithS2CellIdFromPointExample1) {
+  // Failures can be generated for `ConsistentWithS2CellIdFromPoint` by
+  // increasing the number of iterations to 1M.  This is one such example.
+  S2Point p(0.38203141040035632, 0.030196609707941954, 0.9236558700239289);
+  S2Cell cell{S2CellId(p)};
+  EXPECT_TRUE(cell.Contains(p)) << " point: " << p << " cell: " << cell.id();
+}
+
 TEST(S2CellId, AmbiguousContainsPoint) {
   // This tests a case where S2CellId returns the "wrong" cell for a point
   // that is very close to the cell edge. (ConsistentWithS2CellIdFromPoint
@@ -506,11 +540,12 @@ static S1ChordAngle GetMaxDistanceToPointBruteForce(const S2Cell& cell,
 }
 
 TEST(S2Cell, GetDistanceToPoint) {
-  S2Testing::rnd.Reset(absl::GetFlag(FLAGS_s2_random_seed));
+  absl::BitGen bitgen(S2Testing::MakeTaggedSeedSeq(
+      "GET_DISTANCE_TO_POINT", absl::LogInfoStreamer(__FILE__, __LINE__).stream()));
   for (int iter = 0; iter < 1000; ++iter) {
     SCOPED_TRACE(StrCat("Iteration ", iter));
-    S2Cell cell(S2Testing::GetRandomCellId());
-    S2Point target = S2Testing::RandomPoint();
+    S2Cell cell(s2random::CellId(bitgen));
+    S2Point target = s2random::Point(bitgen);
     S1Angle expected_to_boundary =
         GetDistanceToPointBruteForce(cell, target).ToAngle();
     S1Angle expected_to_interior =
@@ -540,22 +575,25 @@ TEST(S2Cell, GetDistanceToPoint) {
   }
 }
 
-static void ChooseEdgeNearCell(const S2Cell& cell, S2Point* a, S2Point* b) {
+static void ChooseEdgeNearCell(absl::BitGenRef bitgen, const S2Cell& cell,
+                               S2Point* a, S2Point* b) {
   S2Cap cap = cell.GetCapBound();
-  if (S2Testing::rnd.OneIn(5)) {
+  if (absl::Bernoulli(bitgen, 0.2)) {
     // Choose a point anywhere on the sphere.
-    *a = S2Testing::RandomPoint();
+    *a = s2random::Point(bitgen);
   } else {
     // Choose a point inside or somewhere near the cell.
-    *a = S2Testing::SamplePoint(S2Cap(cap.center(), 1.5 * cap.GetRadius()));
+    *a = s2random::SamplePoint(bitgen,
+                               S2Cap(cap.center(), 1.5 * cap.GetRadius()));
   }
   // Now choose a maximum edge length ranging from very short to very long
   // relative to the cell size, and choose the other endpoint.
-  double max_length = min(100 * pow(1e-4, S2Testing::rnd.RandDouble()) *
-                          cap.GetRadius().radians(), M_PI_2);
-  *b = S2Testing::SamplePoint(S2Cap(*a, S1Angle::Radians(max_length)));
+  double max_length =
+      min(s2random::LogUniform(bitgen, 1e-2, 1e2) * cap.GetRadius().radians(),
+          M_PI_2);
+  *b = s2random::SamplePoint(bitgen, S2Cap(*a, S1Angle::Radians(max_length)));
 
-  if (S2Testing::rnd.OneIn(20)) {
+  if (absl::Bernoulli(bitgen, 0.05)) {
     // Occasionally replace edge with antipodal edge.
     *a = -*a;
     *b = -*b;
@@ -606,12 +644,13 @@ static S1ChordAngle GetMaxDistanceToEdgeBruteForce(
 }
 
 TEST(S2Cell, GetDistanceToEdge) {
-  S2Testing::rnd.Reset(absl::GetFlag(FLAGS_s2_random_seed));
+  absl::BitGen bitgen(S2Testing::MakeTaggedSeedSeq(
+      "GET_DISTANCE_TO_EDGE", absl::LogInfoStreamer(__FILE__, __LINE__).stream()));
   for (int iter = 0; iter < 1000; ++iter) {
     SCOPED_TRACE(StrCat("Iteration ", iter));
-    S2Cell cell(S2Testing::GetRandomCellId());
+    S2Cell cell(s2random::CellId(bitgen));
     S2Point a, b;
-    ChooseEdgeNearCell(cell, &a, &b);
+    ChooseEdgeNearCell(bitgen, cell, &a, &b);
     S1Angle expected_min = GetDistanceToEdgeBruteForce(cell, a, b).ToAngle();
     S1Angle expected_max =
         GetMaxDistanceToEdgeBruteForce(cell, a, b).ToAngle();
@@ -620,8 +659,8 @@ TEST(S2Cell, GetDistanceToEdge) {
     // The error has a peak near Pi/2 for edge distance, and another peak near
     // Pi for vertex distance.
     if (expected_min.radians() > M_PI/2) {
-      // Max error for S1ChordAngle as it approaches Pi is about 2e-8.
-      EXPECT_NEAR(expected_min.radians(), actual_min.radians(), 2e-8);
+      // Max error for S1ChordAngle as it approaches Pi is about 3e-8.
+      EXPECT_NEAR(expected_min.radians(), actual_min.radians(), 3e-8);
     } else if (expected_min.radians() <= M_PI / 3) {
       EXPECT_NEAR(expected_min.radians(), actual_min.radians(), 1e-15);
     } else {
@@ -658,9 +697,11 @@ TEST(S2Cell, GetMaxDistanceToCellAntipodal) {
 }
 
 TEST(S2Cell, GetMaxDistanceToCell) {
+  absl::BitGen bitgen(S2Testing::MakeTaggedSeedSeq(
+      "GET_MAX_DISTANCE_TO_CELL", absl::LogInfoStreamer(__FILE__, __LINE__).stream()));
   for (int i = 0; i < 1000; i++) {
-    S2Cell cell(S2Testing::GetRandomCellId());
-    S2Cell test_cell(S2Testing::GetRandomCellId());
+    S2Cell cell(s2random::CellId(bitgen));
+    S2Cell test_cell(s2random::CellId(bitgen));
     S2CellId antipodal_leaf_id(-test_cell.GetCenter());
     S2Cell antipodal_test_cell(antipodal_leaf_id.parent(test_cell.level()));
 
@@ -671,6 +712,145 @@ TEST(S2Cell, GetMaxDistanceToCell) {
   }
 }
 
+TEST(S2Cell, IsDistanceLess) {
+  const S2Cell cell1(S2CellId::FromToken("1b"));
+  const S2Cell cell2(S2CellId::FromToken("104"));  // Adjacent to cell1
+  const S2Cell cell3(S2CellId::FromToken("0c"));   // Adjacent to cell1
+  const S2Cell cell4(S2CellId::FromToken("8f"));   // Not adjacent to cell1
+
+  const S1ChordAngle dist12 = cell1.GetDistance(cell2);
+  const S1ChordAngle dist13 = cell1.GetDistance(cell3);
+  const S1ChordAngle dist14 = cell1.GetDistance(cell4);
+
+  const S1ChordAngle kEpsilon = S1ChordAngle::Radians(1e-10);
+
+  ASSERT_EQ(dist12, S1ChordAngle::Zero());
+  ASSERT_EQ(dist13, S1ChordAngle::Zero());
+  ASSERT_GT(dist14, kEpsilon);
+
+  EXPECT_FALSE(cell1.IsDistanceLess(cell2, S1ChordAngle::Zero()));
+  EXPECT_TRUE(cell1.IsDistanceLess(cell2, kEpsilon));
+  EXPECT_FALSE(cell1.IsDistanceLess(cell2, dist12));
+  EXPECT_TRUE(cell1.IsDistanceLess(cell2, dist12 + kEpsilon));
+
+  EXPECT_FALSE(cell1.IsDistanceLess(cell3, S1ChordAngle::Zero()));
+  EXPECT_TRUE(cell1.IsDistanceLess(cell3, kEpsilon));
+  EXPECT_FALSE(cell1.IsDistanceLess(cell3, dist13));
+  EXPECT_TRUE(cell1.IsDistanceLess(cell3, dist13 + kEpsilon));
+
+  EXPECT_FALSE(cell1.IsDistanceLess(cell4, S1ChordAngle::Zero()));
+  EXPECT_FALSE(cell1.IsDistanceLess(cell4, kEpsilon));
+  EXPECT_FALSE(cell1.IsDistanceLess(cell4, dist14 - kEpsilon));
+  EXPECT_FALSE(cell1.IsDistanceLess(cell4, dist14));
+  EXPECT_TRUE(cell1.IsDistanceLess(cell4, dist14 + kEpsilon));
+}
+
+TEST(S2Cell, IsDistanceLessOrEqual) {
+  const S2Cell cell1(S2CellId::FromToken("1b"));
+  const S2Cell cell2(S2CellId::FromToken("104"));  // Adjacent to cell1
+  const S2Cell cell3(S2CellId::FromToken("0c"));   // Adjacent to cell1
+  const S2Cell cell4(S2CellId::FromToken("8f"));   // Not adjacent to cell1
+
+  const S1ChordAngle dist12 = cell1.GetDistance(cell2);
+  const S1ChordAngle dist13 = cell1.GetDistance(cell3);
+  const S1ChordAngle dist14 = cell1.GetDistance(cell4);
+
+  const S1ChordAngle kEpsilon = S1ChordAngle::Radians(1e-10);
+
+  ASSERT_EQ(dist12, S1ChordAngle::Zero());
+  ASSERT_EQ(dist13, S1ChordAngle::Zero());
+  ASSERT_GT(dist14, kEpsilon);
+
+  // IsDistanceLessOrEqual
+  EXPECT_TRUE(cell1.IsDistanceLessOrEqual(cell2, S1ChordAngle::Zero()));
+  EXPECT_TRUE(cell1.IsDistanceLessOrEqual(cell2, kEpsilon));
+  EXPECT_TRUE(cell1.IsDistanceLessOrEqual(cell2, dist12));
+  EXPECT_TRUE(cell1.IsDistanceLessOrEqual(cell2, dist12 + kEpsilon));
+
+  EXPECT_TRUE(cell1.IsDistanceLessOrEqual(cell3, S1ChordAngle::Zero()));
+  EXPECT_TRUE(cell1.IsDistanceLessOrEqual(cell3, kEpsilon));
+  EXPECT_TRUE(cell1.IsDistanceLessOrEqual(cell3, dist13));
+  EXPECT_TRUE(cell1.IsDistanceLessOrEqual(cell3, dist13 + kEpsilon));
+
+  EXPECT_FALSE(cell1.IsDistanceLessOrEqual(cell4, S1ChordAngle::Zero()));
+  EXPECT_FALSE(cell1.IsDistanceLessOrEqual(cell4, kEpsilon));
+  EXPECT_FALSE(cell1.IsDistanceLessOrEqual(cell4, dist14 - kEpsilon));
+  EXPECT_TRUE(cell1.IsDistanceLessOrEqual(cell4, dist14));
+  EXPECT_TRUE(cell1.IsDistanceLessOrEqual(cell4, dist14 + kEpsilon));
+}
+
+TEST(S2Cell, IsMaxDistanceLess) {
+  const S2Cell cell1(S2CellId::FromToken("1b"));
+  const S2Cell cell2(S2CellId::FromToken("104"));  // Adjacent to cell1
+  const S2Cell cell3(S2CellId::FromToken("0c"));   // Adjacent to cell1
+  const S2Cell cell4(S2CellId::FromToken("8f"));   // Not adjacent to cell1
+
+  const S1ChordAngle max_dist12 = cell1.GetMaxDistance(cell2);
+  const S1ChordAngle max_dist13 = cell1.GetMaxDistance(cell3);
+  const S1ChordAngle max_dist14 = cell1.GetMaxDistance(cell4);
+
+  const S1ChordAngle kEpsilon = S1ChordAngle::Radians(1e-10);
+
+  ASSERT_GT(max_dist12, kEpsilon);
+  ASSERT_GT(max_dist13, kEpsilon);
+  ASSERT_GT(max_dist14, kEpsilon);
+
+  // IsMaxDistanceLess
+  EXPECT_FALSE(cell1.IsMaxDistanceLess(cell2, S1ChordAngle::Zero()));
+  EXPECT_FALSE(cell1.IsMaxDistanceLess(cell2, kEpsilon));
+  EXPECT_FALSE(cell1.IsMaxDistanceLess(cell2, max_dist12 - kEpsilon));
+  EXPECT_FALSE(cell1.IsMaxDistanceLess(cell2, max_dist12));
+  EXPECT_TRUE(cell1.IsMaxDistanceLess(cell2, max_dist12 + kEpsilon));
+
+  EXPECT_FALSE(cell1.IsMaxDistanceLess(cell3, S1ChordAngle::Zero()));
+  EXPECT_FALSE(cell1.IsMaxDistanceLess(cell3, kEpsilon));
+  EXPECT_FALSE(cell1.IsMaxDistanceLess(cell3, max_dist13 - kEpsilon));
+  EXPECT_FALSE(cell1.IsMaxDistanceLess(cell3, max_dist13));
+  EXPECT_TRUE(cell1.IsMaxDistanceLess(cell3, max_dist13 + kEpsilon));
+
+  EXPECT_FALSE(cell1.IsMaxDistanceLess(cell4, S1ChordAngle::Zero()));
+  EXPECT_FALSE(cell1.IsMaxDistanceLess(cell4, kEpsilon));
+  EXPECT_FALSE(cell1.IsMaxDistanceLess(cell4, max_dist14 - kEpsilon));
+  EXPECT_FALSE(cell1.IsMaxDistanceLess(cell4, max_dist14));
+  EXPECT_TRUE(cell1.IsMaxDistanceLess(cell4, max_dist14 + kEpsilon));
+}
+
+TEST(S2Cell, IsMaxDistanceLessOrEqual) {
+  const S2Cell cell1(S2CellId::FromToken("1b"));
+  const S2Cell cell2(S2CellId::FromToken("104"));  // Adjacent to cell1
+  const S2Cell cell3(S2CellId::FromToken("0c"));   // Adjacent to cell1
+  const S2Cell cell4(S2CellId::FromToken("8f"));   // Not adjacent to cell1
+
+  const S1ChordAngle max_dist12 = cell1.GetMaxDistance(cell2);
+  const S1ChordAngle max_dist13 = cell1.GetMaxDistance(cell3);
+  const S1ChordAngle max_dist14 = cell1.GetMaxDistance(cell4);
+
+  const S1ChordAngle kEpsilon = S1ChordAngle::Radians(1e-10);
+
+  ASSERT_GT(max_dist12, kEpsilon);
+  ASSERT_GT(max_dist13, kEpsilon);
+  ASSERT_GT(max_dist14, kEpsilon);
+
+  // IsMaxDistanceLessOrEqual
+  EXPECT_FALSE(cell1.IsMaxDistanceLessOrEqual(cell2, S1ChordAngle::Zero()));
+  EXPECT_FALSE(cell1.IsMaxDistanceLessOrEqual(cell2, kEpsilon));
+  EXPECT_FALSE(cell1.IsMaxDistanceLessOrEqual(cell2, max_dist12 - kEpsilon));
+  EXPECT_TRUE(cell1.IsMaxDistanceLessOrEqual(cell2, max_dist12));
+  EXPECT_TRUE(cell1.IsMaxDistanceLessOrEqual(cell2, max_dist12 + kEpsilon));
+
+  EXPECT_FALSE(cell1.IsMaxDistanceLessOrEqual(cell3, S1ChordAngle::Zero()));
+  EXPECT_FALSE(cell1.IsMaxDistanceLessOrEqual(cell3, kEpsilon));
+  EXPECT_FALSE(cell1.IsMaxDistanceLessOrEqual(cell3, max_dist13 - kEpsilon));
+  EXPECT_TRUE(cell1.IsMaxDistanceLessOrEqual(cell3, max_dist13));
+  EXPECT_TRUE(cell1.IsMaxDistanceLessOrEqual(cell3, max_dist13 + kEpsilon));
+
+  EXPECT_FALSE(cell1.IsMaxDistanceLessOrEqual(cell4, S1ChordAngle::Zero()));
+  EXPECT_FALSE(cell1.IsMaxDistanceLessOrEqual(cell4, kEpsilon));
+  EXPECT_FALSE(cell1.IsMaxDistanceLessOrEqual(cell4, max_dist14 - kEpsilon));
+  EXPECT_TRUE(cell1.IsMaxDistanceLessOrEqual(cell4, max_dist14));
+  EXPECT_TRUE(cell1.IsMaxDistanceLessOrEqual(cell4, max_dist14 + kEpsilon));
+}
+
 TEST(S2Cell, EncodeDecode) {
   S2Cell orig_cell(S2LatLng::FromDegrees(40.7406264, -74.0029963));
   Encoder encoder;
@@ -678,7 +858,7 @@ TEST(S2Cell, EncodeDecode) {
 
   S2Cell decoded_cell(S2LatLng::FromDegrees(51.494987, -0.146585));
   Decoder decoder(encoder.base(), encoder.length());
-  decoded_cell.Decode(&decoder);
+  ASSERT_TRUE(decoded_cell.Decode(&decoder));
 
   EXPECT_EQ(orig_cell, decoded_cell);
   EXPECT_EQ(orig_cell.face(), decoded_cell.face());
@@ -688,3 +868,60 @@ TEST(S2Cell, EncodeDecode) {
   EXPECT_EQ(orig_cell.GetBoundUV(), decoded_cell.GetBoundUV());
 }
 
+TEST(S2Cell, GetUVCoordOfEdge) {
+  // Four cells on face 0 with two boundaries each on 0/0.
+  S2Cell kCell0[4] = {
+      S2Cell(S2CellId::FromToken("0f")), S2Cell(S2CellId::FromToken("05")),
+      S2Cell(S2CellId::FromToken("1b")), S2Cell(S2CellId::FromToken("11"))};
+
+  // And four cells on face 4 which is rotated w.r.t face 0.
+  S2Cell kCell4[4] = {
+      S2Cell(S2CellId::FromToken("8f")), S2Cell(S2CellId::FromToken("85")),
+      S2Cell(S2CellId::FromToken("9b")), S2Cell(S2CellId::FromToken("91"))};
+
+  for (int k = 0; k < 4; ++k) {
+    EXPECT_THAT(kCell0[k].GetUVCoordOfEdge(k + 0), Eq(0));
+    EXPECT_THAT(kCell0[k].GetUVCoordOfEdge(k + 1), Eq(0));
+    EXPECT_THAT(kCell4[k].GetUVCoordOfEdge(k + 0), Eq(0));
+    EXPECT_THAT(kCell4[k].GetUVCoordOfEdge(k + 1), Eq(0));
+  }
+}
+
+TEST(S2Cell, GetSizeIJAgreesWithCellId) {
+  absl::BitGen bitgen(S2Testing::MakeTaggedSeedSeq(
+      "GET_SIZE_IJ_AGREES_WITH_CELL_ID", absl::LogInfoStreamer(__FILE__, __LINE__).stream()));
+  for (int i = 0; i < 100; ++i) {
+    S2CellId id = s2random::CellId(bitgen);
+    S2Cell cell(id);
+    EXPECT_EQ(cell.GetSizeIJ(), id.GetSizeIJ(id.level()));
+  }
+}
+
+TEST(S2Cell, GetIJCoordOfEdge) {
+  absl::BitGen bitgen(S2Testing::MakeTaggedSeedSeq(
+      "GET_IJ_COORD_OF_EDGE", absl::LogInfoStreamer(__FILE__, __LINE__).stream()));
+  for (int i = 0; i < 100; ++i) {
+    S2CellId id = s2random::CellId(bitgen);
+    S2Cell cell(id);
+
+    // Look up the canonical IJ coordinates of the cell boundary.
+    int ij[2];
+    int orientation;
+    id.ToFaceIJOrientation(ij, ij + 1, &orientation);
+
+    int ij_size = cell.GetSizeIJ();
+    R2Rect ij_bounds;
+    for (int k = 0; k < 2; ++k) {
+      int ij_lo = ij[k] & -ij_size;
+      ij_bounds[k][0] = ij_lo;
+      ij_bounds[k][1] = ij_lo + ij_size;
+    }
+
+    // Check that each boundary coordinate is correct.
+    for (int k = 0; k < 4; ++k) {
+      EXPECT_THAT(cell.GetIJCoordOfEdge(k),
+                  Eq(ij_bounds.GetVertex(k)[(k + 1) % 2]));
+    }
+  }
+}
+
