@@ -19,35 +19,55 @@
 
 #include <algorithm>
 #include <cmath>
-#include <cstdio>
+#include <cstdint>
 #include <string>
 #include <utility>
 #include <vector>
 
+#include <gmock/gmock.h>
 #include <gtest/gtest.h>
 
+#include "absl/flags/declare.h"
 #include "absl/flags/flag.h"
+#include "absl/hash/hash_testing.h"
+#include "absl/log/absl_check.h"
+#include "absl/log/log_streamer.h"
+#include "absl/random/bit_gen_ref.h"
+#include "absl/random/random.h"
 #include "absl/strings/str_cat.h"
 
-#include "s2/base/integral_types.h"
-#include "s2/base/logging.h"
 #include "s2/s1angle.h"
 #include "s2/s2cap.h"
 #include "s2/s2cell.h"
 #include "s2/s2cell_id.h"
+#include "s2/s2cell_iterator_join.h"
+#include "s2/s2coder_testing.h"
 #include "s2/s2edge_distances.h"
+#include "s2/s2error.h"
 #include "s2/s2metrics.h"
+#include "s2/s2point.h"
+#include "s2/s2random.h"
 #include "s2/s2region_coverer.h"
 #include "s2/s2testing.h"
+#include "s2/s2text_format.h"
 #include "s2/util/coding/coder.h"
 
-S2_DECLARE_bool(s2debug);
+ABSL_DECLARE_FLAG(bool, s2debug);
+
+ABSL_FLAG(int32_t, iters, 2000, "Number of iterations for tests.");
 
 using absl::StrCat;
 using std::max;
 using std::min;
+using std::string;
 using std::vector;
 
+using ::testing::ElementsAre;
+using ::testing::Eq;
+using ::testing::IsEmpty;
+using ::testing::IsFalse;
+using ::testing::IsTrue;
+
 class S2CellUnionTestPeer {
  public:
   // Creates a possibly invalid S2CellUnion without any checks.
@@ -71,7 +91,7 @@ TEST(S2CellUnion, S2CellIdConstructor) {
 
 TEST(S2CellUnion, WholeSphere) {
   S2CellUnion whole_sphere = S2CellUnion::WholeSphere();
-  EXPECT_EQ(whole_sphere.LeafCellsCovered(), 6 * (uint64{1} << 60));
+  EXPECT_EQ(whole_sphere.LeafCellsCovered(), 6 * (uint64_t{1} << 60));
   whole_sphere.Expand(0);
   EXPECT_EQ(whole_sphere, S2CellUnion::WholeSphere());
 }
@@ -116,10 +136,8 @@ TEST(S2CellUnion, IsNormalized) {
   EXPECT_FALSE(cell_union.IsNormalized());
 }
 
-static S2Testing::Random& rnd = S2Testing::rnd;
-
-static void AddCells(S2CellId id, bool selected,
-                     vector<S2CellId> *input, vector<S2CellId> *expected) {
+static void AddCells(absl::BitGenRef bitgen, S2CellId id, bool selected,
+                     vector<S2CellId>* input, vector<S2CellId>* expected) {
   // Decides whether to add "id" and/or some of its descendants to the
   // test case.  If "selected" is true, then the region covered by "id"
   // *must* be added to the test case (either by adding "id" itself, or
@@ -130,37 +148,39 @@ static void AddCells(S2CellId id, bool selected,
   if (id == S2CellId::None()) {
     // Initial call: decide whether to add cell(s) from each face.
     for (int face = 0; face < 6; ++face) {
-      AddCells(S2CellId::FromFace(face), false, input, expected);
+      AddCells(bitgen, S2CellId::FromFace(face), false, input, expected);
     }
     return;
   }
   if (id.is_leaf()) {
-    // The rnd.OneIn() call below ensures that the parent of a leaf cell
-    // will always be selected (if we make it that far down the hierarchy).
-    S2_DCHECK(selected);
+    // The `absl::Bernoulli()` call below ensures that the parent of
+    // a leaf cell will always be selected (if we make it that far down the
+    // hierarchy).
+    ABSL_DCHECK(selected);
     input->push_back(id);
     return;
   }
   // The following code ensures that the probability of selecting a cell
   // at each level is approximately the same, i.e. we test normalization
   // of cells at all levels.
-  if (!selected && rnd.OneIn(S2CellId::kMaxLevel - id.level())) {
+  if (!selected &&
+      absl::Bernoulli(bitgen, 1.0 / (S2CellId::kMaxLevel - id.level()))) {
     // Once a cell has been selected, the expected output is predetermined.
     // We then make sure that cells are selected that will normalize to
     // the desired output.
-    expected->push_back(id);
+    if (expected != nullptr) expected->push_back(id);
     selected = true;
   }
 
-  // With the rnd.OneIn() constants below, this function adds an average
-  // of 5/6 * (kMaxLevel - level) cells to "input" where "level" is the
-  // level at which the cell was first selected (level 15 on average).
+  // With the `absl::Bernoulli()` constants below, this function
+  // adds an average of 5/6 * (kMaxLevel - level) cells to "input" where "level"
+  // is the level at which the cell was first selected (level 15 on average).
   // Therefore the average number of input cells in a test case is about
   // (5/6 * 15 * 6) = 75.  The average number of output cells is about 6.
 
   // If a cell is selected, we add it to "input" with probability 5/6.
   bool added = false;
-  if (selected && !rnd.OneIn(6)) {
+  if (selected && absl::Bernoulli(bitgen, 5.0 / 6)) {
     input->push_back(id);
     added = true;
   }
@@ -175,39 +195,57 @@ static void AddCells(S2CellId id, bool selected,
     // We also make sure that we do not recurse on all 4 children, since
     // then we might include all 4 children in the input case by accident
     // (in which case the expected output would not be correct).
-    if (rnd.OneIn(selected ? 12 : 4) && num_children < 3) {
-      AddCells(child, selected, input, expected);
+    if (absl::Bernoulli(bitgen, 1.0 / (selected ? 12 : 4)) &&
+        num_children < 3) {
+      AddCells(bitgen, child, selected, input, expected);
       ++num_children;
     }
     // If this cell was selected but the cell itself was not added, we
     // must ensure that all 4 children (or some combination of their
     // descendants) are added.
-    if (selected && !added) AddCells(child, selected, input, expected);
+    if (selected && !added) AddCells(bitgen, child, selected, input, expected);
   }
 }
 
-TEST(S2CellUnion, Normalize) {
+TEST(S2CellUnion, AddCellsSizes) {
   // Try a bunch of random test cases, and keep track of average
   // statistics for normalization (to see if they agree with the
   // analysis above).
+  absl::BitGen bitgen(S2Testing::MakeTaggedSeedSeq(
+      "ADD_CELLS_SIZES", absl::LogInfoStreamer(__FILE__, __LINE__).stream()));
   double in_sum = 0, out_sum = 0;
-  static const int kIters = 2000;
-  for (int i = 0; i < kIters; ++i) {
+  const int num_iters = absl::GetFlag(FLAGS_iters);
+  for (int i = 0; i < num_iters; ++i) {
     vector<S2CellId> input, expected;
-    AddCells(S2CellId::None(), false, &input, &expected);
+    AddCells(bitgen, S2CellId::None(), /*selected=*/false, &input, &expected);
     in_sum += input.size();
     out_sum += expected.size();
-    S2CellUnion cellunion(input);
-    EXPECT_EQ(expected.size(), cellunion.size());
-    for (int i = 0; i < expected.size(); ++i) {
-      EXPECT_EQ(expected[i], cellunion[i]);
-    }
+  }
+  absl::PrintF("avg in %.2f, avg out %.2f\n", in_sum / num_iters,
+               out_sum / num_iters);
+}
 
-    // Test GetCapBound().
-    S2Cap cap = cellunion.GetCapBound();
-    for (S2CellId id : cellunion) {
-      EXPECT_TRUE(cap.Contains(S2Cell(id)));
-    }
+TEST(S2CellUnion, ContainsExpectedCells) {
+  absl::BitGen bitgen(S2Testing::MakeTaggedSeedSeq(
+      "CONTAINS_EXPECTED_CELLS", absl::LogInfoStreamer(__FILE__, __LINE__).stream()));
+  const int num_iters = absl::GetFlag(FLAGS_iters);
+  for (int i = 0; i < num_iters; ++i) {
+    vector<S2CellId> input, expected;
+    AddCells(bitgen, S2CellId::None(), /*selected=*/false, &input, &expected);
+    S2CellUnion cellunion(input);
+    EXPECT_THAT(cellunion.cell_ids(), Eq(expected));
+  }
+}
+
+TEST(S2CellUnion, ContainsInputCells) {
+  absl::BitGen bitgen(S2Testing::MakeTaggedSeedSeq(
+      "CONTAINS_INPUT_CELLS", absl::LogInfoStreamer(__FILE__, __LINE__).stream()));
+  const int num_iters = absl::GetFlag(FLAGS_iters);
+  for (int i = 0; i < num_iters; ++i) {
+    vector<S2CellId> input, expected;
+    AddCells(bitgen, S2CellId::None(), /*selected=*/false, &input,
+             /*expected=*/nullptr);
+    S2CellUnion cellunion(input);
 
     // Test Contains(S2CellId) and Intersects(S2CellId).
     for (S2CellId input_id : input) {
@@ -232,19 +270,42 @@ TEST(S2CellUnion, Normalize) {
                         input_id.child_begin(S2CellId::kMaxLevel)));
       }
     }
+  }
+}
+
+TEST(S2CellUnion, DoesNotContainParentsOfExpectedCell) {
+  absl::BitGen bitgen(
+      S2Testing::MakeTaggedSeedSeq("DOES_NOT_CONTAIN_PARENTS_OF_EXPECTED_CELL",
+                              absl::LogInfoStreamer(__FILE__, __LINE__).stream()));
+  const int num_iters = absl::GetFlag(FLAGS_iters);
+  for (int i = 0; i < num_iters; ++i) {
+    vector<S2CellId> input, expected;
+    AddCells(bitgen, S2CellId::None(), /*selected=*/false, &input, &expected);
+    S2CellUnion cellunion(input);
+
     for (S2CellId expected_id : expected) {
       if (!expected_id.is_face()) {
         EXPECT_TRUE(!cellunion.Contains(expected_id.parent()));
         EXPECT_TRUE(!cellunion.Contains(expected_id.parent(0)));
       }
     }
+  }
+}
 
-    // Test Contains(S2CellUnion), Intersects(S2CellUnion), Union(),
-    // Intersection(), and Difference().
-    vector<S2CellId> x, y, x_or_y, x_and_y;
+TEST(S2CellUnion, UnionIsOr) {
+  absl::BitGen bitgen(
+      S2Testing::MakeTaggedSeedSeq("UNION_IS_OR", absl::LogInfoStreamer(__FILE__, __LINE__).stream()));
+  const int num_iters = absl::GetFlag(FLAGS_iters);
+  for (int i = 0; i < num_iters; ++i) {
+    vector<S2CellId> input;
+    AddCells(bitgen, S2CellId::None(), /*selected=*/false, &input,
+             /*expected=*/nullptr);
+    S2CellUnion cellunion(input);
+
+    vector<S2CellId> x, y, x_or_y;
     for (S2CellId input_id : input) {
-      bool in_x = rnd.OneIn(2);
-      bool in_y = rnd.OneIn(2);
+      bool in_x = absl::Bernoulli(bitgen, 0.5);
+      bool in_y = absl::Bernoulli(bitgen, 0.5);
       if (in_x) x.push_back(input_id);
       if (in_y) y.push_back(input_id);
       if (in_x || in_y) x_or_y.push_back(input_id);
@@ -253,7 +314,27 @@ TEST(S2CellUnion, Normalize) {
     S2CellUnion ycells(std::move(y));
     S2CellUnion x_or_y_expected(std::move(x_or_y));
     S2CellUnion x_or_y_cells = xcells.Union(ycells);
-    EXPECT_TRUE(x_or_y_cells == x_or_y_expected);
+    EXPECT_THAT(x_or_y_cells, Eq(x_or_y_expected));
+  }
+}
+
+TEST(S2CellUnion, IntersectionIsAnd) {
+  absl::BitGen bitgen(S2Testing::MakeTaggedSeedSeq(
+      "INTERSECTION_IS_AND", absl::LogInfoStreamer(__FILE__, __LINE__).stream()));
+  const int num_iters = absl::GetFlag(FLAGS_iters);
+  for (int i = 0; i < num_iters; ++i) {
+    vector<S2CellId> input;
+    AddCells(bitgen, S2CellId::None(), /*selected=*/false, &input,
+             /*expected=*/nullptr);
+    S2CellUnion cellunion(input);
+
+    vector<S2CellId> x, y, x_and_y;
+    for (S2CellId input_id : input) {
+      if (absl::Bernoulli(bitgen, 0.5)) x.push_back(input_id);
+      if (absl::Bernoulli(bitgen, 0.5)) y.push_back(input_id);
+    }
+    S2CellUnion xcells(std::move(x));
+    S2CellUnion ycells(std::move(y));
 
     // Compute the intersection of "x" with each cell of "y",
     // check that this intersection is correct, and append the
@@ -262,20 +343,59 @@ TEST(S2CellUnion, Normalize) {
       S2CellUnion ucells = xcells.Intersection(yid);
       for (S2CellId xid : xcells) {
         if (xid.contains(yid)) {
-          EXPECT_TRUE(ucells.size() == 1 && ucells[0] == yid);
+          // TODO(user): Log `input` via `LogContainer`?  This dep
+          // currently only comes in via `compact_array`.
+          EXPECT_THAT(ucells.cell_ids(), ElementsAre(yid))
+              << "xid: " << xid << " yid: " << yid << " xcells: " << xcells
+              << " ycells: " << ycells;
         } else if (yid.contains(xid)) {
-          EXPECT_TRUE(ucells.Contains(xid));
+          EXPECT_TRUE(ucells.Contains(xid))
+              << "xid: " << xid << " yid: " << yid << " xcells: " << xcells
+              << " ycells: " << ycells << " ucells: " << ucells;
         }
       }
       for (S2CellId uid : ucells) {
-        EXPECT_TRUE(xcells.Contains(uid));
-        EXPECT_TRUE(yid.contains(uid));
+        EXPECT_TRUE(xcells.Contains(uid))
+            << "xcells: " << xcells << " uid: " << uid;
+        EXPECT_TRUE(yid.contains(uid))
+            << "xcells: " << s2textformat::ToString(xcells)
+            << " ucells: " << s2textformat::ToString(ucells)
+            << " yid: " << yid.ToToken() << " uid: " << uid.ToToken();
       }
       x_and_y.insert(x_and_y.end(), ucells.begin(), ucells.end());
     }
     S2CellUnion x_and_y_expected(std::move(x_and_y));
     S2CellUnion x_and_y_cells = xcells.Intersection(ycells);
-    EXPECT_TRUE(x_and_y_cells == x_and_y_expected);
+    EXPECT_THAT(x_and_y_cells, Eq(x_and_y_expected));
+  }
+}
+
+TEST(S2CellUnion, IntersectionWithCellIdNotInUnionIsEmpty) {
+  S2CellUnion xcells =
+      s2textformat::MakeCellUnionOrDie("2/1021322000001121003");
+  S2CellId yid = s2textformat::MakeCellIdOrDie("2/10213223");
+  S2CellUnion ucells = xcells.Intersection(yid);
+  EXPECT_THAT(ucells.cell_ids(), IsEmpty());
+  EXPECT_FALSE(ucells.Contains(yid));
+}
+
+TEST(S2CellUnion, DifferenceIsXAndNotY) {
+  absl::BitGen bitgen(S2Testing::MakeTaggedSeedSeq(
+      "DIFFERENCE_IS_X_AND_NOT_Y", absl::LogInfoStreamer(__FILE__, __LINE__).stream()));
+  const int num_iters = absl::GetFlag(FLAGS_iters);
+  for (int i = 0; i < num_iters; ++i) {
+    vector<S2CellId> input;
+    AddCells(bitgen, S2CellId::None(), /*selected=*/false, &input,
+             /*expected=*/nullptr);
+    S2CellUnion cellunion(input);
+
+    vector<S2CellId> x, y;
+    for (S2CellId input_id : input) {
+      if (absl::Bernoulli(bitgen, 0.5)) x.push_back(input_id);
+      if (absl::Bernoulli(bitgen, 0.5)) y.push_back(input_id);
+    }
+    S2CellUnion xcells(std::move(x));
+    S2CellUnion ycells(std::move(y));
 
     S2CellUnion x_minus_y_cells = xcells.Difference(ycells);
     S2CellUnion y_minus_x_cells = ycells.Difference(xcells);
@@ -284,13 +404,51 @@ TEST(S2CellUnion, Normalize) {
     EXPECT_TRUE(ycells.Contains(y_minus_x_cells));
     EXPECT_TRUE(!y_minus_x_cells.Intersects(xcells));
     EXPECT_TRUE(!x_minus_y_cells.Intersects(y_minus_x_cells));
+  }
+}
+
+TEST(S2CellUnion, DifferenceInclusionExclusion) {
+  absl::BitGen bitgen(S2Testing::MakeTaggedSeedSeq(
+      "DIFFERENCE_INCLUSION_EXCLUSION", absl::LogInfoStreamer(__FILE__, __LINE__).stream()));
+  const int num_iters = absl::GetFlag(FLAGS_iters);
+  for (int i = 0; i < num_iters; ++i) {
+    vector<S2CellId> input;
+    AddCells(bitgen, S2CellId::None(), /*selected=*/false, &input,
+             /*expected=*/nullptr);
+    S2CellUnion cellunion(input);
+
+    vector<S2CellId> x, y;
+    for (S2CellId input_id : input) {
+      if (absl::Bernoulli(bitgen, 0.5)) x.push_back(input_id);
+      if (absl::Bernoulli(bitgen, 0.5)) y.push_back(input_id);
+    }
+    S2CellUnion xcells(std::move(x));
+    S2CellUnion ycells(std::move(y));
+
+    S2CellUnion x_minus_y_cells = xcells.Difference(ycells);
+    S2CellUnion y_minus_x_cells = ycells.Difference(xcells);
+    S2CellUnion x_and_y_cells = xcells.Intersection(ycells);
+    S2CellUnion x_or_y_cells = xcells.Union(ycells);
 
     S2CellUnion diff_intersection_union =
         x_minus_y_cells.Union(y_minus_x_cells).Union(x_and_y_cells);
-    EXPECT_TRUE(diff_intersection_union == x_or_y_cells);
+    EXPECT_THAT(diff_intersection_union, Eq(x_or_y_cells));
+  }
+}
 
-    vector<S2CellId> test, dummy;
-    AddCells(S2CellId::None(), false, &test, &dummy);
+TEST(S2CellUnion, ContainsIntersectsBruteForce) {
+  absl::BitGen bitgen(S2Testing::MakeTaggedSeedSeq(
+      "CONTAINS_INTERSECTS_BRUTE_FORCE", absl::LogInfoStreamer(__FILE__, __LINE__).stream()));
+  const int num_iters = absl::GetFlag(FLAGS_iters);
+  for (int i = 0; i < num_iters; ++i) {
+    vector<S2CellId> input, expected;
+    AddCells(bitgen, S2CellId::None(), /*selected=*/false, &input, &expected);
+    S2CellUnion cellunion(input);
+    // Generate some new cells and see if `Contains` / `Intersects` agree
+    // with a brute-force computation.
+    vector<S2CellId> test;
+    AddCells(bitgen, S2CellId::None(), /*selected=*/false, &test,
+             /*expected=*/nullptr);
     for (S2CellId test_id : test) {
       bool contains = false, intersects = false;
       for (S2CellId expected_id : expected) {
@@ -301,7 +459,24 @@ TEST(S2CellUnion, Normalize) {
       EXPECT_EQ(intersects, cellunion.Intersects(test_id));
     }
   }
-  printf("avg in %.2f, avg out %.2f\n", in_sum / kIters, out_sum / kIters);
+}
+
+TEST(S2CellUnion, CapBoundContainsAllCells) {
+  absl::BitGen bitgen(S2Testing::MakeTaggedSeedSeq(
+      "CAP_BOUNDS_CONTAINS_ALL_CELLS", absl::LogInfoStreamer(__FILE__, __LINE__).stream()));
+  const int num_iters = absl::GetFlag(FLAGS_iters);
+  for (int i = 0; i < num_iters; ++i) {
+    vector<S2CellId> input;
+    AddCells(bitgen, S2CellId::None(), /*selected=*/false,  //
+             &input, /*expected=*/nullptr);
+    S2CellUnion cellunion(input);
+    S2Cap cap = cellunion.GetCapBound();
+    for (S2CellId id : cellunion) {
+      EXPECT_TRUE(cap.Contains(S2Cell(id)))
+          << "cap: " << cap << " cellunion: " << cellunion << " id: " << id
+          << " iter: " << i;
+    }
+  }
 }
 
 // Return the maximum geodesic distance from "axis" to any point of
@@ -338,24 +513,29 @@ TEST(S2CellUnion, Expand) {
   // the coverings by a random radius, and then make sure that the new
   // covering covers the expanded cap.  It also makes sure that the
   // new covering is not too much larger than expected.
+  absl::BitGen bitgen(
+      S2Testing::MakeTaggedSeedSeq("EXPAND", absl::LogInfoStreamer(__FILE__, __LINE__).stream()));
 
   S2RegionCoverer coverer;
-  for (int i = 0; i < 1000; ++i) {
+  const int num_iters = absl::GetFlag(FLAGS_iters);
+  for (int i = 0; i < num_iters; ++i) {
     SCOPED_TRACE(StrCat("Iteration ", i));
-    S2Cap cap = S2Testing::GetRandomCap(
-        S2Cell::AverageArea(S2CellId::kMaxLevel), 4 * M_PI);
+    S2Cap cap = s2random::Cap(bitgen, S2Cell::AverageArea(S2CellId::kMaxLevel),
+                              4 * M_PI);
 
     // Expand the cap area by a random factor whose log is uniformly
     // distributed between 0 and log(1e2).
     S2Cap expanded_cap = S2Cap::FromCenterHeight(
-        cap.center(), min(2.0, pow(1e2, rnd.RandDouble()) * cap.height()));
+        cap.center(),
+        min(2.0, s2random::LogUniform(bitgen, 1.0, 1e2) * cap.height()));
 
     double radius = (expanded_cap.GetRadius() - cap.GetRadius()).radians();
-    int max_level_diff = rnd.Uniform(8);
+    int max_level_diff = absl::Uniform(bitgen, 0, 8);
 
     // Generate a covering for the original cap, and measure the maximum
     // distance from the cap center to any point in the covering.
-    coverer.mutable_options()->set_max_cells(1 + rnd.Skewed(10));
+    coverer.mutable_options()->set_max_cells(1 +
+                                             s2random::SkewedInt(bitgen, 10));
     S2CellUnion covering = coverer.GetCovering(cap);
     S2Testing::CheckCovering(cap, covering, true);
     double covering_radius = GetRadius(covering, cap.center());
@@ -433,9 +613,11 @@ TEST(S2CellUnion, FromMinMax) {
   TestFromMinMax(face5_id.range_max(), face5_id.range_max());
 
   // Check random ranges of leaf cells.
+  absl::BitGen bitgen(S2Testing::MakeTaggedSeedSeq(
+      "FROM_MIN_MAX", absl::LogInfoStreamer(__FILE__, __LINE__).stream()));
   for (int iter = 0; iter < 100; ++iter) {
-    S2CellId x = S2Testing::GetRandomCellId(S2CellId::kMaxLevel);
-    S2CellId y = S2Testing::GetRandomCellId(S2CellId::kMaxLevel);
+    S2CellId x = s2random::CellId(bitgen, S2CellId::kMaxLevel);
+    S2CellId y = s2random::CellId(bitgen, S2CellId::kMaxLevel);
     using std::swap;
     if (x > y) swap(x, y);
     TestFromMinMax(x, y);
@@ -554,7 +736,7 @@ TEST(S2CellUnion, Clear) {
 }
 
 TEST(S2CellUnion, RefuseToDecode) {
-  std::vector<S2CellId> cellids;
+  vector<S2CellId> cellids;
   S2CellId id = S2CellId::Begin(S2CellId::kMaxLevel);
   for (int i = 0; i <= absl::GetFlag(FLAGS_s2cell_union_decode_max_num_cells);
        ++i) {
@@ -589,18 +771,18 @@ TEST(S2CellUnion, LeafCellsCovered) {
   // One leaf cell on face 0.
   ids.push_back(S2CellId::FromFace(0).child_begin(S2CellId::kMaxLevel));
   cell_union.Init(ids);
-  EXPECT_EQ(uint64{1}, cell_union.LeafCellsCovered());
+  EXPECT_EQ(uint64_t{1}, cell_union.LeafCellsCovered());
 
   // Face 0 itself (which includes the previous leaf cell).
   ids.push_back(S2CellId::FromFace(0));
   cell_union.Init(ids);
-  EXPECT_EQ(uint64{1} << 60, cell_union.LeafCellsCovered());
+  EXPECT_EQ(uint64_t{1} << 60, cell_union.LeafCellsCovered());
   // Five faces.
   cell_union.Expand(0);
-  EXPECT_EQ(uint64{5} << 60, cell_union.LeafCellsCovered());
+  EXPECT_EQ(uint64_t{5} << 60, cell_union.LeafCellsCovered());
   // Whole world.
   cell_union.Expand(0);
-  EXPECT_EQ(uint64{6} << 60, cell_union.LeafCellsCovered());
+  EXPECT_EQ(uint64_t{6} << 60, cell_union.LeafCellsCovered());
 
   // Add some disjoint cells.
   ids.push_back(S2CellId::FromFace(1).child_begin(1));
@@ -611,8 +793,8 @@ TEST(S2CellUnion, LeafCellsCovered) {
   ids.push_back(S2CellId::FromFace(4).child_end(15).prev());
   ids.push_back(S2CellId::FromFace(5).child_begin(30));
   cell_union.Init(ids);
-  uint64 expected = 1ULL + (1ULL << 6) + (1ULL << 30) + (1ULL << 32) +
-      (2ULL << 56) + (1ULL << 58) + (1ULL << 60);
+  uint64_t expected = 1ULL + (1ULL << 6) + (1ULL << 30) + (1ULL << 32) +
+                      (2ULL << 56) + (1ULL << 58) + (1ULL << 60);
   EXPECT_EQ(expected, cell_union.LeafCellsCovered());
 }
 
@@ -646,7 +828,7 @@ TEST(S2CellUnion, ToStringTwoCells) {
 TEST(S2CellUnion, ToStringOver500Cells) {
   vector<S2CellId> ids;
   S2CellUnion({S2CellId::FromFace(1)}).Denormalize(6, 1, &ids);  // 4096 cells
-  std::string result = S2CellUnion::FromVerbatim(ids).ToString();
+  string result = S2CellUnion::FromVerbatim(ids).ToString();
   EXPECT_EQ(std::count(result.begin(), result.end(), ','), 500);
   EXPECT_EQ(result.substr(result.size() - 4), ",...");
 }
@@ -660,3 +842,38 @@ TEST(S2CellUnion, IntersectionOneInputNormalized) {
   EXPECT_EQ(intersection, children);
 }
 
+TEST(S2CellUnion, SupportsAbslHash) {
+  EXPECT_TRUE(absl::VerifyTypeImplementsAbslHashCorrectly(
+      {S2CellUnion({S2CellId::FromFace(1)}),
+       S2CellUnion({S2CellId::FromFace(1), S2CellId::FromFace(2).child_begin(1),
+                    S2CellId::FromFace(3).child_begin(2)})}));
+}
+
+TEST(S2CellUnion, IteratorWorks) {
+  // Get a cell union of the face cells.
+  auto cell_union = S2CellUnion::FromBeginEnd(
+      S2CellId::Begin(S2CellId::kMaxLevel), S2CellId::End(S2CellId::kMaxLevel));
+  ABSL_CHECK_EQ(cell_union.num_cells(), 6);
+
+  auto iter = S2CellUnion::Iterator(&cell_union);
+
+  EXPECT_THAT(iter.id(), Eq(S2CellId::FromFace(0)));
+  EXPECT_THAT(iter.done(), IsFalse());
+  EXPECT_THAT(iter.Prev(), IsFalse());
+
+  iter.Next();
+  EXPECT_THAT(iter.id(), Eq(S2CellId::FromFace(1)));
+
+  iter.Finish();
+  EXPECT_THAT(iter.done(), IsTrue());
+  EXPECT_THAT(iter.Prev(), IsTrue());
+  EXPECT_THAT(iter.id(), Eq(S2CellId::FromFace(5)));
+
+  iter.Seek(S2CellId::FromFace(3));
+  EXPECT_THAT(iter.done(), IsFalse());
+  EXPECT_THAT(iter.id(), Eq(S2CellId::FromFace(3)));
+
+  iter.Begin();
+  EXPECT_THAT(iter.done(), IsFalse());
+  EXPECT_THAT(iter.id(), Eq(S2CellId::FromFace(0)));
+}
