@@ -20,20 +20,30 @@
 #ifndef S2_S2CLOSEST_CELL_QUERY_BASE_H_
 #define S2_S2CLOSEST_CELL_QUERY_BASE_H_
 
+#include <algorithm>
+#include <cstddef>
+#include <iterator>
+#include <limits>
+#include <queue>
+#include <type_traits>
 #include <vector>
 
-#include "s2/base/logging.h"
 #include "absl/container/btree_set.h"
+#include "absl/container/flat_hash_set.h"
 #include "absl/container/inlined_vector.h"
 #include "absl/hash/hash.h"
+#include "absl/log/absl_check.h"
+#include "absl/log/absl_log.h"
+#include "s2/_fp_contract_off.h"  // IWYU pragma: keep
 #include "s2/s1chord_angle.h"
 #include "s2/s2cap.h"
+#include "s2/s2cell.h"
 #include "s2/s2cell_id.h"
 #include "s2/s2cell_index.h"
 #include "s2/s2cell_union.h"
 #include "s2/s2distance_target.h"
+#include "s2/s2region.h"
 #include "s2/s2region_coverer.h"
-#include "s2/util/gtl/dense_hash_set.h"
 
 // S2ClosestCellQueryBase is a templatized class for finding the closest
 // (cell_id, label) pairs in an S2CellIndex to a given target.  It is not
@@ -328,12 +338,7 @@ class S2ClosestCellQueryBase {
   // TODO(ericv): Check whether it is faster to avoid duplicates by default
   // (even when Options::max_results() == 1), rather than just when we need to.
   bool avoid_duplicates_;
-  struct LabelledCellHash {
-    size_t operator()(LabelledCell x) const {
-      return absl::HashOf(x.cell_id.id(), x.label);
-    }
-  };
-  gtl::dense_hash_set<LabelledCell, LabelledCellHash> tested_cells_;
+  absl::flat_hash_set<LabelledCell> tested_cells_;
 
   // The algorithm maintains a priority queue of unprocessed S2CellIds, sorted
   // in increasing order of distance from the target.
@@ -372,10 +377,8 @@ class S2ClosestCellQueryBase {
 
 //////////////////   Implementation details follow   ////////////////////
 
-
 template <class Distance>
-inline S2ClosestCellQueryBase<Distance>::Options::Options() {
-}
+inline S2ClosestCellQueryBase<Distance>::Options::Options() = default;
 
 template <class Distance>
 inline int S2ClosestCellQueryBase<Distance>::Options::max_results() const {
@@ -385,7 +388,7 @@ inline int S2ClosestCellQueryBase<Distance>::Options::max_results() const {
 template <class Distance>
 inline void S2ClosestCellQueryBase<Distance>::Options::set_max_results(
     int max_results) {
-  S2_DCHECK_GE(max_results, 1);
+  ABSL_DCHECK_GE(max_results, 1);
   max_results_ = max_results;
 }
 
@@ -438,9 +441,7 @@ inline void S2ClosestCellQueryBase<Distance>::Options::set_use_brute_force(
 
 template <class Distance>
 S2ClosestCellQueryBase<Distance>::S2ClosestCellQueryBase()
-    : tested_cells_(1) /* expected_max_elements*/ {
-  tested_cells_.set_empty_key(LabelledCell(S2CellId::None(), -1));
-}
+    : tested_cells_(/*bucket_count=*/1) {}
 
 template <class Distance>
 S2ClosestCellQueryBase<Distance>::~S2ClosestCellQueryBase() {
@@ -485,7 +486,7 @@ template <class Distance>
 typename S2ClosestCellQueryBase<Distance>::Result
 S2ClosestCellQueryBase<Distance>::FindClosestCell(
     Target* target, const Options& options) {
-  S2_DCHECK_EQ(options.max_results(), 1);
+  ABSL_DCHECK_EQ(options.max_results(), 1);
   FindClosestCellsInternal(target, options);
   return result_singleton_;
 }
@@ -520,16 +521,16 @@ void S2ClosestCellQueryBase<Distance>::FindClosestCellsInternal(
   contents_it_.Clear();
   distance_limit_ = options.max_distance();
   result_singleton_ = Result();
-  S2_DCHECK(result_vector_.empty());
-  S2_DCHECK(result_set_.empty());
-  S2_DCHECK_GE(target->max_brute_force_index_size(), 0);
+  ABSL_DCHECK(result_vector_.empty());
+  ABSL_DCHECK(result_set_.empty());
+  ABSL_DCHECK_GE(target->max_brute_force_index_size(), 0);
   if (distance_limit_ == Distance::Zero()) return;
 
   if (options.max_results() == Options::kMaxMaxResults &&
       options.max_distance() == Distance::Infinity() &&
       options.region() == nullptr) {
-    S2_LOG(WARNING) << "Returning all cells "
-                    "(max_results/max_distance/region not set)";
+    ABSL_LOG(WARNING) << "Returning all cells "
+                         "(max_results/max_distance/region not set)";
   }
 
   // If max_error() > 0 and the target takes advantage of this, then we may
@@ -590,10 +591,7 @@ void S2ClosestCellQueryBase<Distance>::FindClosestCellsOptimized() {
     // it before adding any new entries to the queue.
     QueueEntry entry = queue_.top();
     queue_.pop();
-    // Work around weird parse error in gcc 4.9 by using a local variable for
-    // entry.distance.
-    Distance distance = entry.distance;
-    if (!(distance < distance_limit_)) {
+    if (!(entry.distance < distance_limit_)) {
       queue_ = CellQueue();  // Clear any remaining entries.
       break;
     }
@@ -611,7 +609,7 @@ void S2ClosestCellQueryBase<Distance>::FindClosestCellsOptimized() {
 
 template <class Distance>
 void S2ClosestCellQueryBase<Distance>::InitQueue() {
-  S2_DCHECK(queue_.empty());
+  ABSL_DCHECK(queue_.empty());
 
   // Optimization: rather than starting with the entire index, see if we can
   // limit the search region to a small disc.  Then we can find a covering for
@@ -672,7 +670,7 @@ void S2ClosestCellQueryBase<Distance>::InitQueue() {
     initial_cells = &intersection_with_max_distance_;
   }
   NonEmptyRangeIterator range(index_);
-  for (int i = 0; i < initial_cells->size(); ++i) {
+  for (size_t i = 0; i < initial_cells->size(); ++i) {
     S2CellId id = (*initial_cells)[i];
     bool seek = (i == 0) || id.range_min() >= range.limit_id();
     ProcessOrEnqueue(id, &range, seek);
@@ -738,7 +736,7 @@ void S2ClosestCellQueryBase<Distance>::AddInitialRange(
     S2CellId first_id, S2CellId last_id) {
   // Add the lowest common ancestor of the given range.
   int level = first_id.GetCommonAncestorLevel(last_id);
-  S2_DCHECK_GE(level, 0);
+  ABSL_DCHECK_GE(level, 0);
   index_covering_.push_back(first_id.parent(level));
 }
 
@@ -748,7 +746,7 @@ template <class Distance>
 void S2ClosestCellQueryBase<Distance>::MaybeAddResult(S2CellId cell_id,
                                                       Label label) {
   if (avoid_duplicates_ &&
-      !tested_cells_.insert(LabelledCell(cell_id, label)).second) {
+      !tested_cells_.insert(LabelledCell{cell_id, label}).second) {
     return;
   }
 
