@@ -16,30 +16,42 @@
 // Author: ericv@google.com (Eric Veach)
 
 #include "s2/s2edge_crossings.h"
-#include "s2/s2edge_crossings_internal.h"
 
+#include <algorithm>
 #include <cmath>
-#include <cmath>
+#include <cstdint>
+#include <ios>
+#include <limits>
+#include <utility>
+
+#include "absl/base/casts.h"
+#include "absl/base/optimization.h"
+#include "absl/log/absl_check.h"
+#include "absl/log/absl_log.h"
+#include "absl/strings/string_view.h"
 
-#include "s2/base/logging.h"
 #include "s2/s1angle.h"
 #include "s2/s2edge_crosser.h"
+#include "s2/s2edge_crossings_internal.h"
+#include "s2/s2point.h"
 #include "s2/s2pointutil.h"
 #include "s2/s2predicates.h"
+#include "s2/s2predicates_internal.h"
 #include "s2/util/math/exactfloat/exactfloat.h"
 
 namespace S2 {
 
+using absl::string_view;
+using exactfloat::ExactFloat;
 using internal::GetIntersectionExact;
-using internal::IntersectionMethod;
 using internal::intersection_method_tally_;
+using internal::IntersectionMethod;
 using S2::internal::GetStableCrossProd;
 using s2pred::DBL_ERR;
-using s2pred::ToLD;
-using s2pred::ToExact;
-using s2pred::kHasLongDouble;
 using s2pred::kSqrt3;
 using s2pred::rounding_epsilon;
+using s2pred::ToExact;
+using s2pred::ToLD;
 using std::fabs;
 using std::max;
 using std::sqrt;
@@ -68,7 +80,7 @@ const S1Angle kIntersectionExactError = S1Angle::Radians(2 * DBL_ERR);
 
 int* intersection_method_tally_ = nullptr;
 
-const char* GetIntersectionMethodName(IntersectionMethod method) {
+string_view GetIntersectionMethodName(IntersectionMethod method) {
   switch (method) {
     case IntersectionMethod::SIMPLE:    return "Simple";
     case IntersectionMethod::SIMPLE_LD: return "Simple_ld";
@@ -111,7 +123,11 @@ inline bool GetStableCrossProd(const Vector3<T>& a, const Vector3<T>& b,
   // "a" and "b" are closer than about 18 * DBL_ERR == 9 * DBL_EPSILON.
   // (80-bit precision can handle inputs as close as 2.5 * LDBL_EPSILON.)
   constexpr T T_ERR = rounding_epsilon<T>();
-  constexpr T kMinNorm =
+  // `kMinNorm` should be `constexpr`, but we make it only `const` to work
+  // around a gcc ppc64el bug with `long double`s.
+  // https://gcc.gnu.org/bugzilla/show_bug.cgi?id=107745
+  // https://github.com/google/s2geometry/issues/279
+  const T kMinNorm =
       (32 * kSqrt3 * DBL_ERR) /
       (kRobustCrossProdError.radians() / T_ERR - (1 + 2 * kSqrt3));
 
@@ -122,15 +138,15 @@ inline bool GetStableCrossProd(const Vector3<T>& a, const Vector3<T>& b,
 // Explicitly instantiate this function so that we can use it in tests without
 // putting its definition in a header file.
 template bool GetStableCrossProd<double>(
-    const S2Point&, const S2Point&, S2Point*);
+  const Vector3_d&, const Vector3_d&, Vector3_d*);
 template bool GetStableCrossProd<long double>(
     const Vector3_ld&, const Vector3_ld&, Vector3_ld*);
 
 }  // namespace internal
 
-Vector3_d RobustCrossProd(const S2Point& a, const S2Point& b) {
-  S2_DCHECK(IsUnitLength(a));
-  S2_DCHECK(IsUnitLength(b));
+S2Point RobustCrossProd(const S2Point& a, const S2Point& b) {
+  ABSL_DCHECK(IsUnitLength(a));
+  ABSL_DCHECK(IsUnitLength(b));
 
   // The direction of a.CrossProd(b) becomes unstable as (a + b) or (a - b)
   // approaches zero.  This leads to situations where a.CrossProd(b) is not
@@ -149,9 +165,11 @@ Vector3_d RobustCrossProd(const S2Point& a, const S2Point& b) {
   if (a == b) {
     return Ortho(a);
   }
+  constexpr bool kUseLongDoubleInRobustCrossProd = s2pred::kHasLongDouble;
   // Next we try using "long double" precision (if available).
   Vector3_ld result_ld;
-  if (kHasLongDouble && GetStableCrossProd(ToLD(a), ToLD(b), &result_ld)) {
+  if (kUseLongDoubleInRobustCrossProd &&
+      GetStableCrossProd(ToLD(a), ToLD(b), &result_ld)) {
     return Vector3_d::Cast(result_ld);
   }
   // Otherwise we fall back to exact arithmetic, then symbolic perturbations.
@@ -163,8 +181,8 @@ Vector3_d RobustCrossProd(const S2Point& a, const S2Point& b) {
 // collinear, e.g. if a == -b or a == (1+eps) * b.)  The result may not be
 // normalizable (i.e., EnsureNormalizable() should be called on the result).
 static Vector3_d SymbolicCrossProdSorted(const S2Point& a, const S2Point& b) {
-  S2_DCHECK(a < b);
-  S2_DCHECK(s2pred::IsZero(ToExact(a).CrossProd(ToExact(b))));
+  ABSL_DCHECK(a < b);
+  ABSL_DCHECK(s2pred::IsZero(ToExact(a).CrossProd(ToExact(b))));
 
   // The following code uses the same symbolic perturbation model as S2::Sign.
   // The particular sequence of tests below was obtained using Mathematica
@@ -228,7 +246,7 @@ static Vector3_d SymbolicCrossProdSorted(const S2Point& a, const S2Point& b) {
   // None of the remaining cases can occur in practice, because we can only get
   // to this point if b = (0, 0, 0).  Nevertheless, even (0, 0, 0) has a
   // well-defined direction under the symbolic perturbation model.
-  S2_DCHECK(b[1] == 0 && b[2] == 0);        // da[0] coefficients (always zero)
+  ABSL_DCHECK(b[1] == 0 && b[2] == 0);  // da[0] coefficients (always zero)
 
   if (a[0] != 0 || a[1] != 0) {          // db[2]
     return Vector3_d(a[1], -a[0], 0);
@@ -274,7 +292,7 @@ inline static bool IsNormalizable(const Vector3_d& p) {
 //
 // REQUIRES: p != (0, 0, 0)
 inline static Vector3_d EnsureNormalizable(const Vector3_d& p) {
-  S2_DCHECK_NE(p, Vector3_d(0, 0, 0));
+  ABSL_DCHECK_NE(p, Vector3_d(0, 0, 0));
   if (!IsNormalizable(p)) {
     // We can't just scale by a fixed factor because the smallest representable
     // double is 2**-1074, so if we multiplied by 2**(1074 - 242) then the
@@ -298,7 +316,7 @@ inline static Vector3_d EnsureNormalizable(const Vector3_d& p) {
 // call Normalize() since that would create additional error in situations
 // where normalization is not necessary.)
 static Vector3_d NormalizableFromExact(const Vector3_xf& xf) {
-  Vector3_d x(xf[0].ToDouble(), xf[1].ToDouble(), xf[2].ToDouble());
+  Vector3_d x = Vector3_d::Cast(xf);
   if (IsNormalizable(x)) {
     return x;
   }
@@ -307,20 +325,20 @@ static Vector3_d NormalizableFromExact(const Vector3_xf& xf) {
   // representable by an IEEE double precision float.
   int exp = ExactFloat::kMinExp - 1;
   for (int i = 0; i < 3; ++i) {
-    if (xf[i].is_normal()) exp = std::max(exp, xf[i].exp());
+    if (isnormal(xf[i])) exp = std::max(exp, xf[i].exp());
   }
   if (exp < ExactFloat::kMinExp) {
     return Vector3_d(0, 0, 0);  // The exact result is (0, 0, 0).
   }
-  return Vector3_d(ldexp(xf[0], -exp).ToDouble(),
-                   ldexp(xf[1], -exp).ToDouble(),
-                   ldexp(xf[2], -exp).ToDouble());
+  return Vector3_d(static_cast<double>(ldexp(xf[0], -exp)),
+                   static_cast<double>(ldexp(xf[1], -exp)),
+                   static_cast<double>(ldexp(xf[2], -exp)));
 }
 
 namespace internal {
 
 Vector3_d SymbolicCrossProd(const S2Point& a, const S2Point& b) {
-  S2_DCHECK_NE(a, b);
+  ABSL_DCHECK_NE(a, b);
   // SymbolicCrossProdSorted() requires that a < b.
   if (a < b) {
     return EnsureNormalizable(SymbolicCrossProdSorted(a, b));
@@ -329,7 +347,7 @@ Vector3_d SymbolicCrossProd(const S2Point& a, const S2Point& b) {
   }
 }
 Vector3_d ExactCrossProd(const S2Point& a, const S2Point& b) {
-  S2_DCHECK_NE(a, b);
+  ABSL_DCHECK_NE(a, b);
   Vector3_xf result_xf = ToExact(a).CrossProd(ToExact(b));
   if (!s2pred::IsZero(result_xf)) {
     return NormalizableFromExact(result_xf);
@@ -368,7 +386,7 @@ bool VertexCrossing(const S2Point& a, const S2Point& b,
   if (a == d) return (b == c) || s2pred::OrderedCCW(S2::RefDir(a), c, b, a);
   if (b == c) return s2pred::OrderedCCW(S2::RefDir(b), d, a, b);
 
-  S2_LOG(DFATAL) << "VertexCrossing called with 4 distinct vertices";
+  ABSL_LOG(DFATAL) << "VertexCrossing called with 4 distinct vertices";
   return false;
 }
 
@@ -389,8 +407,8 @@ int SignedVertexCrossing(const S2Point& a, const S2Point& b,
   }
   if (b == c) return s2pred::OrderedCCW(S2::RefDir(b), d, a, b) ? -1 : 0;
 
-  S2_LOG(DFATAL) << "SignedVertexCrossing called with 4 distinct vertices";
-  return false;
+  ABSL_LOG(DFATAL) << "SignedVertexCrossing called with 4 distinct vertices";
+  return 0;
 }
 
 bool EdgeOrVertexCrossing(const S2Point& a, const S2Point& b,
@@ -458,7 +476,7 @@ static bool GetIntersectionSimple(const Vector3<T>& a0, const Vector3<T>& a1,
       12 / (kIntersectionError.radians() / T_ERR - (2 + 2 * kSqrt3));
 
   // On some platforms "long double" is the same as "double", and on these
-  // platforms this method always returns false (e.g. ARM, Win32).  Rather
+  // platforms this method always returns false (e.g. ARM32, Win32).  Rather
   // than testing this directly, instead we look at kMinResultLen since this
   // is a direct measure of whether "long double" has sufficient accuracy to
   // be useful.  If kMinResultLen >= 0.5, it means that this method will fail
@@ -541,7 +559,7 @@ template <class T>
 static bool GetIntersectionStableSorted(
     const Vector3<T>& a0, const Vector3<T>& a1,
     const Vector3<T>& b0, const Vector3<T>& b1, Vector3<T>* result) {
-  S2_DCHECK_GE((a1 - a0).Norm2(), (b1 - b0).Norm2());
+  ABSL_DCHECK_GE((a1 - a0).Norm2(), (b1 - b0).Norm2());
 
   // Compute the normal of the plane through (a0, a1) in a stable way.
   Vector3<T> a_norm = (a0 - a1).CrossProd(a0 + a1);
@@ -602,18 +620,6 @@ static bool GetIntersectionStableSorted(
   return true;
 }
 
-// Returns whether (a0,a1) is less than (b0,b1) with respect to a total
-// ordering on edges that is invariant under edge reversals.
-template <class T>
-static bool CompareEdges(const Vector3<T>& a0, const Vector3<T>& a1,
-                         const Vector3<T>& b0, const Vector3<T>& b1) {
-  const Vector3<T> *pa0 = &a0, *pa1 = &a1;
-  const Vector3<T> *pb0 = &b0, *pb1 = &b1;
-  if (*pa0 >= *pa1) std::swap(pa0, pa1);
-  if (*pb0 >= *pb1) std::swap(pb0, pb1);
-  return *pa0 < *pb0 || (*pa0 == *pb0 && *pb0 < *pb1);
-}
-
 // If the intersection point of the edges (a0,a1) and (b0,b1) can be computed
 // to within an error of at most kIntersectionError by this function, then set
 // "result" to the intersection point and return true.
@@ -630,16 +636,23 @@ static bool GetIntersectionStable(const Vector3<T>& a0, const Vector3<T>& a1,
   //    is used for interpolation (where a shorter edge means less error).
   T a_len2 = (a1 - a0).Norm2();
   T b_len2 = (b1 - b0).Norm2();
-  if (a_len2 < b_len2 || (a_len2 == b_len2 && CompareEdges(a0, a1, b0, b1))) {
+  if (a_len2 < b_len2 ||
+      (a_len2 == b_len2 && internal::CompareEdges(a0, a1, b0, b1))) {
     return GetIntersectionStableSorted(b0, b1, a0, a1, result);
   } else {
     return GetIntersectionStableSorted(a0, a1, b0, b1, result);
   }
 }
 
-static bool GetIntersectionStableLD(const S2Point& a0, const S2Point& a1,
-                                    const S2Point& b0, const S2Point& b1,
-                                    S2Point* result) {
+inline static S2Point ToS2Point(const Vector3_xf& xf) {
+  return NormalizableFromExact(xf).Normalize();
+}
+
+namespace internal {
+
+bool GetIntersectionStableLD(const S2Point& a0, const S2Point& a1,
+                             const S2Point& b0, const S2Point& b1,
+                             S2Point* result) {
   Vector3_ld result_ld;
   if (GetIntersectionStable(ToLD(a0), ToLD(a1), ToLD(b0), ToLD(b1),
                             &result_ld)) {
@@ -649,12 +662,6 @@ static bool GetIntersectionStableLD(const S2Point& a0, const S2Point& a1,
   return false;
 }
 
-inline static S2Point ToS2Point(const Vector3_xf& xf) {
-  return NormalizableFromExact(xf).Normalize();
-}
-
-namespace internal {
-
 // Compute the intersection point of (a0, a1) and (b0, b1) using exact
 // arithmetic.  Note that the result is not exact because it is rounded to
 // double precision.
@@ -696,7 +703,7 @@ S2Point GetIntersectionExact(const S2Point& a0, const S2Point& a1,
   if (s2pred::OrderedCCW(a0, b0, a1, a_norm) && b0 < x) x = b0;
   if (s2pred::OrderedCCW(a0, b1, a1, a_norm) && b1 < x) x = b1;
 
-  S2_DCHECK(S2::IsUnitLength(x));
+  ABSL_DCHECK(S2::IsUnitLength(x));
   return x;
 }
 
@@ -716,7 +723,7 @@ static bool ApproximatelyOrdered(const S2Point& a, const S2Point& x,
 
 S2Point GetIntersection(const S2Point& a0, const S2Point& a1,
                         const S2Point& b0, const S2Point& b1) {
-  S2_DCHECK_GT(CrossingSign(a0, a1, b0, b1), 0);
+  ABSL_DCHECK_GT(CrossingSign(a0, a1, b0, b1), 0);
 
   // It is difficult to compute the intersection point of two edges accurately
   // when the angle between the edges is very small.  Previously we handled
@@ -743,27 +750,33 @@ S2Point GetIntersection(const S2Point& a0, const S2Point& a1,
   //
   // We don't actually use the first method (GetIntersectionSimple) because it
   // turns out that GetIntersectionStable() is twice as fast and also much
-  // more accurate (even in double precision).  The "long double" version
-  // (only available on some platforms) uses 80-bit precision and is about
-  // twice as slow.  The exact arithmetic version is about 100x slower.
+  // more accurate (even in double precision). The "long double" version
+  // (only available on some platforms) uses 80-bit precision on x64 and
+  // 128-bit precision on ARM64, and is ~7x slower on x86. The exact arithmetic
+  // version is about 140x slower than "double" on x86.
   //
   // So our strategy is to first call GetIntersectionStable() in double
-  // precision; if that doesn't work and this platform supports "long double",
-  // then we try again in "long double"; if that doesn't work then we fall
-  // back to exact arithmetic.
+  // precision; if that doesn't work then we fall back to exact arithmetic.
+  // Because "long double" version gives different results depending on the
+  // platform, it is not used in this function.
 
+  // TODO(b/365080041): consider moving the unused implementations to the test
+  // so they can be benchmarked and compared for accuracy.
   constexpr bool kUseSimpleMethod = false;
+  // Long double version produces different results on x86-64 and ARM64
+  // platforms, and is not used in this function.
+  constexpr bool kUseLongDoubleInIntersection = s2pred::kHasLongDouble && false;
   S2Point result;
   IntersectionMethod method;
   if (kUseSimpleMethod && GetIntersectionSimple(a0, a1, b0, b1, &result)) {
     method = IntersectionMethod::SIMPLE;
-  } else if (kUseSimpleMethod && kHasLongDouble &&
+  } else if (kUseSimpleMethod && kUseLongDoubleInIntersection &&
              GetIntersectionSimpleLD(a0, a1, b0, b1, &result)) {
     method = IntersectionMethod::SIMPLE_LD;
   } else if (GetIntersectionStable(a0, a1, b0, b1, &result)) {
     method = IntersectionMethod::STABLE;
-  } else if (kHasLongDouble &&
-             GetIntersectionStableLD(a0, a1, b0, b1, &result)) {
+  } else if (kUseLongDoubleInIntersection &&
+             internal::GetIntersectionStableLD(a0, a1, b0, b1, &result)) {
     method = IntersectionMethod::STABLE_LD;
   } else {
     result = GetIntersectionExact(a0, a1, b0, b1);
@@ -774,8 +787,14 @@ S2Point GetIntersection(const S2Point& a0, const S2Point& a1,
   }
 
   // Make sure that the intersection point lies on both edges.
-  S2_DCHECK(ApproximatelyOrdered(a0, result, a1, kIntersectionError.radians()));
-  S2_DCHECK(ApproximatelyOrdered(b0, result, b1, kIntersectionError.radians()));
+  ABSL_DCHECK(
+      ApproximatelyOrdered(a0, result, a1, kIntersectionError.radians()))
+      << "Using method " << static_cast<int>(method) << " for intersection of "
+      << std::hexfloat << a0 << ", " << a1 << ", " << b0 << ", " << b1;
+  ABSL_DCHECK(
+      ApproximatelyOrdered(b0, result, b1, kIntersectionError.radians()))
+      << "Using method " << static_cast<int>(method) << " for intersection of "
+      << std::hexfloat << a0 << ", " << a1 << ", " << b0 << ", " << b1;
 
   return result;
 }
