@@ -15,36 +15,50 @@
 
 #include "s2/s2furthest_edge_query.h"
 
+#include <algorithm>
+#include <cmath>
 #include <memory>
-#include <set>
+#include <string>
+#include <utility>
 #include <vector>
 
 #include <gtest/gtest.h>
 
 #include "absl/flags/flag.h"
-#include "absl/memory/memory.h"
+#include "absl/log/absl_check.h"
+#include "absl/log/log_streamer.h"
+#include "absl/random/bit_gen_ref.h"
+#include "absl/random/random.h"
+#include "absl/types/span.h"
 
 #include "s2/mutable_s2shape_index.h"
 #include "s2/s1angle.h"
+#include "s2/s1chord_angle.h"
 #include "s2/s2cap.h"
 #include "s2/s2cell.h"
 #include "s2/s2cell_id.h"
 #include "s2/s2closest_edge_query_testing.h"
-#include "s2/s2coords.h"
 #include "s2/s2edge_distances.h"
-#include "s2/s2loop.h"
+#include "s2/s2latlng.h"
+#include "s2/s2max_distance_targets.h"
 #include "s2/s2metrics.h"
 #include "s2/s2point.h"
+#include "s2/s2point_vector_shape.h"
+#include "s2/s2pointutil.h"
 #include "s2/s2polygon.h"
 #include "s2/s2predicates.h"
+#include "s2/s2random.h"
+#include "s2/s2shape.h"
+#include "s2/s2shapeutil_count_edges.h"
+#include "s2/s2shapeutil_shape_edge_id.h"
 #include "s2/s2testing.h"
 #include "s2/s2text_format.h"
 
-using absl::make_unique;
 using s2textformat::MakeIndexOrDie;
 using s2textformat::MakePointOrDie;
 using s2textformat::ParsePointsOrDie;
 using std::make_pair;
+using std::make_unique;
 using std::min;
 using std::pair;
 using std::unique_ptr;
@@ -58,6 +72,8 @@ TEST(S2FurthestEdgeQuery, NoEdges) {
   EXPECT_EQ(S1ChordAngle::Negative(), edge.distance());
   EXPECT_EQ(-1, edge.edge_id());
   EXPECT_EQ(-1, edge.shape_id());
+  EXPECT_FALSE(edge.is_interior());
+  EXPECT_TRUE(edge.is_empty());
   EXPECT_EQ(S1ChordAngle::Negative(), query.GetDistance(&target));
 }
 
@@ -82,6 +98,24 @@ TEST(S2FurthestEdgeQuery, OptionsNotModified) {
   EXPECT_EQ(options.max_error(), query.options().max_error());
 }
 
+TEST(S2FurthestEdgeQuery, OptionsS1AngleSetters) {
+  // Verify that the S1Angle and S1ChordAngle versions do the same thing.
+  // This is mainly to prevent the (so far unused) S1Angle versions from
+  // being detected as dead code.
+  S2FurthestEdgeQuery::Options angle_options, chord_angle_options;
+  angle_options.set_min_distance(S1Angle::Degrees(1));
+  chord_angle_options.set_min_distance(S1ChordAngle::Degrees(1));
+  EXPECT_EQ(chord_angle_options.min_distance(), angle_options.min_distance());
+
+  angle_options.set_inclusive_min_distance(S1Angle::Degrees(1));
+  chord_angle_options.set_inclusive_min_distance(S1ChordAngle::Degrees(1));
+  EXPECT_EQ(chord_angle_options.min_distance(), angle_options.min_distance());
+
+  angle_options.set_conservative_min_distance(S1Angle::Degrees(1));
+  chord_angle_options.set_conservative_min_distance(S1ChordAngle::Degrees(1));
+  EXPECT_EQ(chord_angle_options.min_distance(), angle_options.min_distance());
+}
+
 // In furthest edge queries, the following distance computation is used when
 // updating max distances.
 S1ChordAngle GetMaxDistanceToEdge(
@@ -159,6 +193,8 @@ TEST(S2FurthestEdgeQuery, AntipodalPointInsideIndexedPolygon) {
   EXPECT_EQ(1, results[0].shape_id());
   // Should find the interior, so no specific edge id.
   EXPECT_EQ(-1, results[0].edge_id());
+  EXPECT_TRUE(results[0].is_interior());
+  EXPECT_FALSE(results[0].is_empty());
 
   // Next check that with include_interiors set to false, the distance is less
   // than 180 for the same target and index.
@@ -168,7 +204,14 @@ TEST(S2FurthestEdgeQuery, AntipodalPointInsideIndexedPolygon) {
   EXPECT_LE(results[0].distance(), S1ChordAngle::Straight());
   EXPECT_EQ(1, results[0].shape_id());
   // Found a specific edge, so id should be positive.
-  EXPECT_NE(-1, results[0].edge_id());
+  EXPECT_EQ(3, results[0].edge_id());
+  EXPECT_FALSE(results[0].is_interior());
+  EXPECT_FALSE(results[0].is_empty());
+  S2Shape::Edge e0 = query.GetEdge(results[0]);
+  EXPECT_TRUE(S2::ApproxEquals(e0.v0, S2LatLng::FromDegrees(5, 10).ToPoint()))
+      << S2LatLng(e0.v0);
+  EXPECT_TRUE(S2::ApproxEquals(e0.v1, S2LatLng::FromDegrees(0, 10).ToPoint()))
+      << S2LatLng(e0.v1);
 }
 
 TEST(S2FurthestEdgeQuery, AntipodalPointOutsideIndexedPolygon) {
@@ -301,6 +344,19 @@ TEST(S2FurthestEdgeQuery, FullS2PolygonTarget) {
   EXPECT_EQ(S1ChordAngle::Straight(), full_query.GetDistance(&target));
 }
 
+TEST(S2FurthestEdgeQuery, CheckSettings) {
+  auto full_polygon_index = MakeIndexOrDie("# #");
+  full_polygon_index->Add(make_unique<S2Polygon::OwningShape>(
+      s2textformat::MakePolygonOrDie("full")));
+
+  S2FurthestEdgeQuery::ShapeIndexTarget target(full_polygon_index.get());
+  target.set_include_interiors(true);
+  target.set_use_brute_force(true);
+
+  EXPECT_TRUE(target.include_interiors());
+  EXPECT_TRUE(target.use_brute_force());
+}
+
 //////////////////////////////////////////////////////////////////////////////
 //  General query testing by comparing with brute force method.
 //////////////////////////////////////////////////////////////////////////////
@@ -315,7 +371,7 @@ using Result = pair<S2MaxDistance, s2shapeutil::ShapeEdgeId>;
 // of the following code may become redundant with that in
 // s2closest_edge_query.cc.
 vector<Result> ConvertResults(
-    const vector<S2FurthestEdgeQuery::Result>& edges) {
+    absl::Span<const S2FurthestEdgeQuery::Result> edges) {
   vector<Result> results;
   for (const auto& edge : edges) {
     results.push_back(
@@ -395,20 +451,18 @@ static void TestFindFurthestEdges(
 // (Note that every query is checked using the brute force algorithm.)
 static void TestWithIndexFactory(const s2testing::ShapeIndexFactory& factory,
                                  int num_indexes, int num_edges,
-                                 int num_queries) {
+                                 int num_queries, absl::BitGenRef bitgen) {
   // Build a set of MutableS2ShapeIndexes containing the desired geometry.
   vector<S2Cap> index_caps;
   vector<unique_ptr<MutableS2ShapeIndex>> indexes;
   for (int i = 0; i < num_indexes; ++i) {
-    S2Testing::rnd.Reset(absl::GetFlag(FLAGS_s2_random_seed) + i);
-    index_caps.push_back(S2Cap(S2Testing::RandomPoint(), kTestCapRadius));
+    index_caps.push_back(S2Cap(s2random::Point(bitgen), kTestCapRadius));
     indexes.emplace_back(new MutableS2ShapeIndex);
     factory.AddEdges(index_caps.back(), num_edges, indexes.back().get());
   }
 
   for (int i = 0; i < num_queries; ++i) {
-    S2Testing::rnd.Reset(absl::GetFlag(FLAGS_s2_random_seed) + i);
-    int i_index = S2Testing::rnd.Uniform(num_indexes);
+    int i_index = absl::Uniform(bitgen, 0, num_indexes);
     const S2Cap& index_cap = index_caps[i_index];
 
     // Choose query points from an area approximately 4x larger than the
@@ -417,75 +471,83 @@ static void TestWithIndexFactory(const s2testing::ShapeIndexFactory& factory,
     S2FurthestEdgeQuery query(indexes[i_index].get());
 
     // Exercise the opposite-hemisphere code 1/5 of the time.
-    int antipodal = S2Testing::rnd.OneIn(5) ? -1 : 1;
+    int antipodal = absl::Bernoulli(bitgen, 0.2) ? -1 : 1;
     S2Cap query_cap(antipodal * index_cap.center(), query_radius);
 
     // Occasionally we don't set any limit on the number of result edges.
     // (This may return all edges if we also don't set a distance limit.)
-    if (!S2Testing::rnd.OneIn(5)) {
-      query.mutable_options()->set_max_results(1 + S2Testing::rnd.Uniform(10));
+    if (absl::Bernoulli(bitgen, 0.8)) {
+      query.mutable_options()->set_max_results(absl::Uniform(bitgen, 1, 11));
     }
     // We set a distance limit 2/3 of the time.
-    if (!S2Testing::rnd.OneIn(3)) {
+    if (!absl::Bernoulli(bitgen, 1.0 / 3)) {
       query.mutable_options()->set_min_distance(
-          S2Testing::rnd.RandDouble() * query_radius);
+          absl::Uniform(bitgen, 0.0, 1.0) * query_radius);
     }
-    if (S2Testing::rnd.OneIn(2)) {
+    if (absl::Bernoulli(bitgen, 0.5)) {
       // Choose a maximum error whose logarithm is uniformly distributed over
       // a reasonable range, except that it is sometimes zero.
       query.mutable_options()->set_max_error(S1Angle::Radians(
-          pow(1e-4, S2Testing::rnd.RandDouble()) * query_radius.radians()));
+          s2random::LogUniform(bitgen, 1e-4, 1.0) * query_radius.radians()));
     }
-    query.mutable_options()->set_include_interiors(S2Testing::rnd.OneIn(2));
-    int target_type = S2Testing::rnd.Uniform(4);
+    query.mutable_options()->set_include_interiors(
+        absl::Bernoulli(bitgen, 0.5));
+    int target_type = absl::Uniform(bitgen, 0, 4);
     if (target_type == 0) {
       // Find the edges furthest from a given point.
-      S2Point point = S2Testing::SamplePoint(query_cap);
+      S2Point point = s2random::SamplePoint(bitgen, query_cap);
       S2FurthestEdgeQuery::PointTarget target(point);
       TestFindFurthestEdges(&target, &query);
     } else if (target_type == 1) {
       // Find the edges furthest from a given edge.
-      S2Point a = S2Testing::SamplePoint(query_cap);
-      S2Point b = S2Testing::SamplePoint(
-          S2Cap(a, pow(1e-4, S2Testing::rnd.RandDouble()) * query_radius));
+      S2Point a = s2random::SamplePoint(bitgen, query_cap);
+      S2Point b = s2random::SamplePoint(
+          bitgen,
+          S2Cap(a, s2random::LogUniform(bitgen, 1e-4, 1.0) * query_radius));
       S2FurthestEdgeQuery::EdgeTarget target(a, b);
       TestFindFurthestEdges(&target, &query);
     } else if (target_type == 2) {
       // Find the edges furthest from a given cell.
       int min_level = S2::kMaxDiag.GetLevelForMaxValue(query_radius.radians());
-      int level = min_level + S2Testing::rnd.Uniform(
-          S2CellId::kMaxLevel - min_level + 1);
-      S2Point a = S2Testing::SamplePoint(query_cap);
+      int level = absl::Uniform(absl::IntervalClosedClosed, bitgen, min_level,
+                                S2CellId::kMaxLevel);
+      S2Point a = s2random::SamplePoint(bitgen, query_cap);
       S2Cell cell(S2CellId(a).parent(level));
       S2FurthestEdgeQuery::CellTarget target(cell);
       TestFindFurthestEdges(&target, &query);
     } else {
-      S2_DCHECK_EQ(3, target_type);
+      ABSL_DCHECK_EQ(3, target_type);
       // Use another one of the pre-built indexes as the target.
-      int j_index = S2Testing::rnd.Uniform(num_indexes);
+      int j_index = absl::Uniform(bitgen, 0, num_indexes);
       S2FurthestEdgeQuery::ShapeIndexTarget target(indexes[j_index].get());
-      target.set_include_interiors(S2Testing::rnd.OneIn(2));
+      target.set_include_interiors(absl::Bernoulli(bitgen, 0.5));
       TestFindFurthestEdges(&target, &query);
     }
   }
 }
 
-static const int kNumIndexes = 50;
-static const int kNumEdges = 100;
-static const int kNumQueries = 200;
+static constexpr int kNumIndexes = 50;
+static constexpr int kNumEdges = 100;
+static constexpr int kNumQueries = 200;
 
 TEST(S2FurthestEdgeQuery, CircleEdges) {
-  TestWithIndexFactory(s2testing::RegularLoopShapeIndexFactory(),
-                       kNumIndexes, kNumEdges, kNumQueries);
+  absl::BitGen bitgen(S2Testing::MakeTaggedSeedSeq(
+      "CIRCLE_EDGES", absl::LogInfoStreamer(__FILE__, __LINE__).stream()));
+  TestWithIndexFactory(s2testing::RegularLoopShapeIndexFactory(), kNumIndexes,
+                       kNumEdges, kNumQueries, bitgen);
 }
 
 TEST(S2FurthestEdgeQuery, FractalEdges) {
-  TestWithIndexFactory(s2testing::FractalLoopShapeIndexFactory(),
-                       kNumIndexes, kNumEdges, kNumQueries);
+  absl::BitGen bitgen(S2Testing::MakeTaggedSeedSeq(
+      "FRACTAL_EDGES", absl::LogInfoStreamer(__FILE__, __LINE__).stream()));
+  TestWithIndexFactory(s2testing::FractalLoopShapeIndexFactory(bitgen),
+                       kNumIndexes, kNumEdges, kNumQueries, bitgen);
 }
 
 TEST(S2FurthestEdgeQuery, PointCloudEdges) {
-  TestWithIndexFactory(s2testing::PointCloudShapeIndexFactory(),
-                       kNumIndexes, kNumEdges, kNumQueries);
+  absl::BitGen bitgen(S2Testing::MakeTaggedSeedSeq(
+      "POINT_CLOUD_EDGES", absl::LogInfoStreamer(__FILE__, __LINE__).stream()));
+  TestWithIndexFactory(s2testing::PointCloudShapeIndexFactory(bitgen),
+                       kNumIndexes, kNumEdges, kNumQueries, bitgen);
 }
 
