@@ -17,37 +17,46 @@
 #include "s2/s2polyline_alignment.h"
 
 #include <algorithm>
-#include <numeric>
+#include <cmath>
+#include <memory>
+#include <ostream>
 #include <sstream>
 #include <string>
 #include <utility>
 #include <vector>
 
-#include "s2/base/logging.h"
-#include "absl/memory/memory.h"
+#include "absl/log/absl_check.h"
+#include "absl/types/span.h"
+#include "s2/s2point.h"
+#include "s2/s2polyline.h"
 #include "s2/s2polyline_alignment_internal.h"
-#include "s2/util/math/mathutil.h"
 
+using std::make_unique;
 using std::string;
+using std::unique_ptr;
+using std::vector;
 
 namespace s2polyline_alignment {
 
-Window::Window(const std::vector<ColumnStride>& strides) {
-  S2_DCHECK(!strides.empty()) << "Cannot construct empty window.";
-  S2_DCHECK(strides[0].start == 0) << "First element of start_cols is non-zero.";
+Window::Window(const vector<ColumnStride>& strides) {
+  ABSL_DCHECK(!strides.empty()) << "Cannot construct empty window.";
+  ABSL_DCHECK(strides[0].start == 0)
+      << "First element of start_cols is non-zero.";
   strides_ = strides;
   rows_ = strides.size();
   cols_ = strides.back().end;
-  S2_DCHECK(this->IsValid()) << "Constructor validity check fail.";
+  ABSL_DCHECK(this->IsValid()) << "Constructor validity check fail.";
 }
 
 Window::Window(const WarpPath& warp_path) {
-  S2_DCHECK(!warp_path.empty()) << "Cannot construct window from empty warp path.";
-  S2_DCHECK(warp_path.front() == std::make_pair(0, 0)) << "Must start at (0, 0).";
+  ABSL_DCHECK(!warp_path.empty())
+      << "Cannot construct window from empty warp path.";
+  ABSL_DCHECK(warp_path.front() == std::make_pair(0, 0))
+      << "Must start at (0, 0).";
   rows_ = warp_path.back().first + 1;
-  S2_DCHECK(rows_ > 0) << "Must have at least one row.";
+  ABSL_DCHECK(rows_ > 0) << "Must have at least one row.";
   cols_ = warp_path.back().second + 1;
-  S2_DCHECK(cols_ > 0) << "Must have at least one column.";
+  ABSL_DCHECK(cols_ > 0) << "Must have at least one column.";
   strides_.resize(rows_);
 
   int prev_row = 0;
@@ -63,17 +72,17 @@ Window::Window(const WarpPath& warp_path) {
     }
     stride_stop = pair.second + 1;
   }
-  S2_DCHECK_EQ(curr_row, rows_ - 1);
+  ABSL_DCHECK_EQ(curr_row, rows_ - 1);
   strides_[rows_ - 1] = {stride_start, stride_stop};
-  S2_DCHECK(this->IsValid()) << "Constructor validity check fail.";
+  ABSL_DCHECK(this->IsValid()) << "Constructor validity check fail.";
 }
 
 Window Window::Upsample(const int new_rows, const int new_cols) const {
-  S2_DCHECK(new_rows >= rows_) << "Upsampling: New_rows < current_rows";
-  S2_DCHECK(new_cols >= cols_) << "Upsampling: New_cols < current_cols";
+  ABSL_DCHECK(new_rows >= rows_) << "Upsampling: New_rows < current_rows";
+  ABSL_DCHECK(new_cols >= cols_) << "Upsampling: New_cols < current_cols";
   const double row_scale = static_cast<double>(new_rows) / rows_;
   const double col_scale = static_cast<double>(new_cols) / cols_;
-  std::vector<ColumnStride> new_strides(new_rows);
+  vector<ColumnStride> new_strides(new_rows);
   ColumnStride from_stride;
   for (int row = 0; row < new_rows; ++row) {
     from_stride = strides_[static_cast<int>((row + 0.5) / row_scale)];
@@ -91,8 +100,8 @@ Window Window::Upsample(const int new_rows, const int new_cols) const {
 // straightforward to do so. This method generally isn't very expensive so it
 // feels unnecessary to combine them.
 Window Window::Dilate(const int radius) const {
-  S2_DCHECK(radius >= 0) << "Negative dilation radius.";
-  std::vector<ColumnStride> new_strides(rows_);
+  ABSL_DCHECK(radius >= 0) << "Negative dilation radius.";
+  vector<ColumnStride> new_strides(rows_);
   int prev_row, next_row;
   for (int row = 0; row < rows_; ++row) {
     prev_row = std::max(0, row - radius);
@@ -179,18 +188,22 @@ VertexAlignment DynamicTimewarp(const S2Polyline& a, const S2Polyline& b,
                                 const Window& w) {
   const int rows = a.num_vertices();
   const int cols = b.num_vertices();
-  auto costs = CostTable(rows, std::vector<double>(cols));
+  auto costs = CostTable(rows, vector<double>(cols));
 
   ColumnStride curr;
   ColumnStride prev = ColumnStride::All();
   for (int row = 0; row < rows; ++row) {
     curr = w.GetColumnStride(row);
     for (int col = curr.start; col < curr.end; ++col) {
+      // The total cost up to (row,col) is the minimum of the cost up, down,
+      // left and the distance between the points in row and col. We use
+      // the distance between the points, as we are trying to minimize the
+      // distance between the two polylines.
       double d_cost = BoundsCheckedTableCost(row - 1, col - 1, prev, costs);
       double u_cost = BoundsCheckedTableCost(row - 1, col - 0, prev, costs);
       double l_cost = BoundsCheckedTableCost(row - 0, col - 1, curr, costs);
       costs[row][col] = std::min({d_cost, u_cost, l_cost}) +
-                        (a.vertex(row) - b.vertex(col)).Norm2();
+                        (a.vertex(row) - b.vertex(col)).Norm();
     }
     prev = curr;
   }
@@ -232,14 +245,14 @@ VertexAlignment DynamicTimewarp(const S2Polyline& a, const S2Polyline& b,
   return VertexAlignment(costs.back().back(), warp_path);
 }
 
-std::unique_ptr<S2Polyline> HalfResolution(const S2Polyline& in) {
+unique_ptr<S2Polyline> HalfResolution(const S2Polyline& in) {
   const int n = in.num_vertices();
-  std::vector<S2Point> vertices;
+  vector<S2Point> vertices;
   vertices.reserve(n / 2);
   for (int i = 0; i < n; i += 2) {
     vertices.push_back(in.vertex(i));
   }
-  return absl::make_unique<S2Polyline>(vertices);
+  return make_unique<S2Polyline>(vertices);
 }
 
 // Helper methods for GetMedoidPolyline and GetConsensusPolyline to auto-select
@@ -262,15 +275,15 @@ VertexAlignment AlignmentFn(const S2Polyline& a, const S2Polyline& b,
 double GetExactVertexAlignmentCost(const S2Polyline& a, const S2Polyline& b) {
   const int a_n = a.num_vertices();
   const int b_n = b.num_vertices();
-  S2_CHECK(a_n > 0) << "A is empty polyline.";
-  S2_CHECK(b_n > 0) << "B is empty polyline.";
-  std::vector<double> cost(b_n, DOUBLE_MAX);
+  ABSL_CHECK(a_n > 0) << "A is empty polyline.";
+  ABSL_CHECK(b_n > 0) << "B is empty polyline.";
+  vector<double> cost(b_n, DOUBLE_MAX);
   double left_diag_min_cost = 0;
   for (int row = 0; row < a_n; ++row) {
     for (int col = 0; col < b_n; ++col) {
       double up_cost = cost[col];
       cost[col] = std::min(left_diag_min_cost, up_cost) +
-                  (a.vertex(row) - b.vertex(col)).Norm2();
+                  (a.vertex(row) - b.vertex(col)).Norm();
       left_diag_min_cost = std::min(cost[col], up_cost);
     }
     left_diag_min_cost = DOUBLE_MAX;
@@ -282,9 +295,9 @@ VertexAlignment GetExactVertexAlignment(const S2Polyline& a,
                                         const S2Polyline& b) {
   const int a_n = a.num_vertices();
   const int b_n = b.num_vertices();
-  S2_CHECK(a_n > 0) << "A is empty polyline.";
-  S2_CHECK(b_n > 0) << "B is empty polyline.";
-  const auto w = Window(std::vector<ColumnStride>(a_n, {0, b_n}));
+  ABSL_CHECK(a_n > 0) << "A is empty polyline.";
+  ABSL_CHECK(b_n > 0) << "B is empty polyline.";
+  const auto w = Window(vector<ColumnStride>(a_n, {0, b_n}));
   return DynamicTimewarp(a, b, w);
 }
 
@@ -298,9 +311,9 @@ VertexAlignment GetApproxVertexAlignment(const S2Polyline& a,
   const double kDensitySwitchover = 0.85;
   const int a_n = a.num_vertices();
   const int b_n = b.num_vertices();
-  S2_CHECK(a_n > 0) << "A is empty polyline.";
-  S2_CHECK(b_n > 0) << "B is empty polyline.";
-  S2_CHECK(radius >= 0) << "Radius is negative.";
+  ABSL_CHECK(a_n > 0) << "A is empty polyline.";
+  ABSL_CHECK(b_n > 0) << "B is empty polyline.";
+  ABSL_CHECK(radius >= 0) << "Radius is negative.";
 
   // If we've hit the point where doing a full, direct solve is guaranteed to
   // be faster, then terminate the recursion and do that.
@@ -336,14 +349,14 @@ VertexAlignment GetApproxVertexAlignment(const S2Polyline& a,
 // alignments. Specifically, because cost_fn(a, b) = cost_fn(b, a), and
 // cost_fn(a, a) = 0, we can compute only the lower triangle of cost matrix
 // and then mirror it across the diagonal to save on cost_fn invocations.
-int GetMedoidPolyline(const std::vector<std::unique_ptr<S2Polyline>>& polylines,
+int GetMedoidPolyline(absl::Span<const unique_ptr<S2Polyline>> polylines,
                       const MedoidOptions options) {
   const int num_polylines = polylines.size();
   const bool approx = options.approx();
-  S2_CHECK_GT(num_polylines, 0);
+  ABSL_CHECK_GT(num_polylines, 0);
 
   // costs[i] stores total cost of aligning [i] with all other polylines.
-  std::vector<double> costs(num_polylines, 0.0);
+  vector<double> costs(num_polylines, 0.0);
   for (int i = 0; i < num_polylines; ++i) {
     for (int j = i + 1; j < num_polylines; ++j) {
       double cost = CostFn(*polylines[i], *polylines[j], approx);
@@ -373,11 +386,11 @@ int GetMedoidPolyline(const std::vector<std::unique_ptr<S2Polyline>>& polylines,
 //
 //  This algorithm takes O(iteration_cap * num_polylines) pairwise alignments.
 
-std::unique_ptr<S2Polyline> GetConsensusPolyline(
-    const std::vector<std::unique_ptr<S2Polyline>>& polylines,
+unique_ptr<S2Polyline> GetConsensusPolyline(
+    absl::Span<const unique_ptr<S2Polyline>> polylines,
     const ConsensusOptions options) {
   const int num_polylines = polylines.size();
-  S2_CHECK_GT(num_polylines, 0);
+  ABSL_CHECK_GT(num_polylines, 0);
   const bool approx = options.approx();
 
   // Seed a consensus polyline, either arbitrarily with first element, or with
@@ -388,14 +401,14 @@ std::unique_ptr<S2Polyline> GetConsensusPolyline(
     medoid_options.set_approx(approx);
     seed_index = GetMedoidPolyline(polylines, medoid_options);
   }
-  auto consensus = std::unique_ptr<S2Polyline>(polylines[seed_index]->Clone());
+  auto consensus = unique_ptr<S2Polyline>(polylines[seed_index]->Clone());
   const int num_consensus_vertices = consensus->num_vertices();
-  S2_DCHECK_GT(num_consensus_vertices, 1);
+  ABSL_DCHECK_GT(num_consensus_vertices, 1);
 
   bool converged = false;
   int iterations = 0;
   while (!converged && iterations < options.iteration_cap()) {
-    std::vector<S2Point> points(num_consensus_vertices, S2Point());
+    vector<S2Point> points(num_consensus_vertices, S2Point());
     for (const auto& polyline : polylines) {
       const auto alignment = AlignmentFn(*consensus, *polyline, approx);
       for (const auto& pair : alignment.warp_path) {
@@ -407,7 +420,7 @@ std::unique_ptr<S2Polyline> GetConsensusPolyline(
     }
 
     ++iterations;
-    auto new_consensus = absl::make_unique<S2Polyline>(points);
+    auto new_consensus = make_unique<S2Polyline>(points);
     converged = new_consensus->ApproxEquals(*consensus);
     consensus = std::move(new_consensus);
   }
