@@ -18,7 +18,31 @@
 #include "s2/s2shape_index.h"
 
 #include <gtest/gtest.h>
+#include "s2/s2shape.h"
+#include "s2/s2text_format.h"
 
 // TODO(ericv): Add tests for S2ShapeIndexCell and S2ClippedShape.
 // (Currently these are tested indirectly by MutableS2ShapeIndex.)
 // Also test the base Iterator type (which wraps another iterator).
+
+TEST(S2ShapeIndexIterator, PrefixIncrement) {
+  const auto index =
+      s2textformat::MakeIndexOrDie("1:1 # 1:1, 2:2 # 2:2, 1:3, 2:4, 3:2");
+
+  int count = 0;
+  for (auto iter = index->begin(); iter != index->end(); ++iter) {
+    ++count;
+  }
+  EXPECT_EQ(count, 3);
+}
+
+TEST(S2ShapeIndexIterator, PostfixIncrement) {
+  const auto index =
+      s2textformat::MakeIndexOrDie("1:1 # 1:1, 2:2 # 2:2, 1:3, 2:4, 3:2");
+
+  int count = 0;
+  for (auto iter = index->begin(); iter != index->end(); iter++) {
+    ++count;
+  }
+  EXPECT_EQ(count, 3);
+}
